#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName = HITRAN
#pragma version = 1.1

#include "LineBroadening" menus=0, version>=1.1

// HITRAN Procedures
// 
// Procedures for loading data from the HITRAN database into Igor and running atmospheric absorption/transmission calculations.  
// To download the database, go to:
//
//  http://www.cfa.harvard.edu/hitran/
//
//  and register for access to the database (It's free but they request you fill out a questionaire on usage)
//  For a quick start, you'll need the files molparam.txt, parsumdata.txt and at least one line file (these are
//  *.par files but better to download as a *.zip.  HITRAN08.zip contains the whole database and 
//  is quite large.  Run "HITRAN->Setup->HITRAN initial Setup" to load these files and save them
//   locally as Igor packed binary files, and
//  HITRAN->Setup->HITRAN Package Start" to create new experiment files after the first setup.  
// 
//  See included help file "HITRANhelp.ihf" for more information.  
//
//  Ian Konen
//  iankonen@gmail.com
//
//
//
// Revision History
// Version 1.0 released 11/29/12
// 
// Version 1.1 
//  Simplified Setup Process:  
//      New Function HITRANAutoSetup() created to selectively launch the first time setup (SetUpHITRANPackage()) or package setup
//      with already converted binaries (HT_InitializePackage()).  User can select option from a pull down menu, but the default choice is 
//	  guessed at by existance or lack thereof of package preference file.  Should be a little less hassle.  If you're a heavy HITRAN user (who isnt?)
//      and use a loading procedure to load your favorite packages, you might want to include this initialization function.  I.E....I have the following line
//      in a file in my Igor Procedures folder so the setup runs everytime I load the HITRAN procedure file:

//       Menu "Other Packages"
//		"HITRAN Procs", Execute/P "INSERTINCLUDE \"HITRAN\"";Execute/P "COMPILEPROCEDURES ";Execute/P "HITRAN#HITRANAutoSetup() "
//	   End
//
//
//  Changed Calculation Advisor to a panel & added line width histograms and atmosphere population check.
//
//  Changed Fast Calculaton Fill Algorithm to use built-in weighted histogram operation.
// 
//  Molecule Color Code panel is now a control panel allowing the user to select new colors for each molecule
//
//  Suppress Line Broadening menus (what can I say, I just noticed the menus=0 option for #includes :-)) 
//    If you want the Line Broadening menus back (perhaps to convolve a HITRAN simulation with an instrument function)
//    you can modify the #include line in this file and remove the menus=0 option (will get overwritten if you update HITRAN.ipf) or 
//    #include "LineBroadening" in the local procedure file for each experiment that needs it (the better way to do it).  
//


//#define ORIG    // uncomment to alter text wave copy method in MakeSubParFolder(...)



// Scientific Constants:

Static Constant PRESSTODENS = 7.336095539848891e21    // (Density in molecules / cm^3) = (pressure in atm) / (temp in K) * PRESSTODENS
Static Constant KAYINWN = 0.69503476 			// boltzman's constant  (cm-1 / K)        	(from wikipediea)
Static Constant KAYINJOULES = 1.3806488e-23  	// boltzman's constant  (J / K) 			(from wikipediea)
Static Constant AMU2KG = 1.66053886E-27			// 1000 / Avogadro's number, or conversion factor of mass in amu to mass in kg (of a single particle not a mol!)
Static Constant CINMS = 2.99792458e8				// Speed of Light in m/s
Static Constant GSIGTOFWHM = 2.35482004503094932701 // = 2sqrt(2ln(2)) to convert Gaussian standard deviation to Full Width Half Max
Static Constant HINWN = 3.33645e-11				// Planck's Constant in wavenumber seconds, or just inverse speed of light (in s / cm)
Static Constant STANDARDTEMP = 296			// Kelvin
Static Constant STANDARDPRESS = 1				// atm

////2008 version -- not really using these much.  
//
//Makes more sense to count numbers from molparam.txt to accommidate future versions.
// useful reference though.

Static Constant NUMMOLECULES = 42
Static Constant NUMISOTOPES = 100
Static Constant NUMTEMPS = 2931
Static StrConstant HITRANVERS = "Hitran08"
Static StrConstant CT_FOR_MOLECULES = "SpectrumBlack"		// Sets the Color Table used for molecule labeling in stick spectra and atmospheric selector.

Static StrConstant HT_DF = "root:Packages:HITRAN:"
Static StrConstant HT_PARDF = "root:Packages:HITRAN:PARSets:"

Constant DEFAULT_FILL_METHOD = 1		// See LineBroadening.ipf.  Fill Method for "FillDestWave" function.  Set to 0 for older method 
										// by modifying this line or overriding in experiment procedure file.

Menu "HITRAN"
		SubMenu "SetUp"
			"HITRAN Start", HITRAN#HITRANAutoSetup()
//			"HITRAN Setup From Database Files", HITRAN#SetUpHITRANPackage()
//			"HITRAN Initialize Package Globals", HITRAN#HT_InitializePackage()
			"Set Package Preferences", HITRAN#HT_SetPreferences()
			"Load PAR File", HITRAN#HT_LoadParFileGUI()
			"Save PAR Binary", HITRAN#HT_SaveParBinary()
		End
		SubMenu "Parameter Sets"
			"Create Subset", HITRAN#HT_SelectSpeciesGUI()
			"Combine Parameter Sets", HITRAN#HT_CombineParFoldersMenu()
			"Delete Parameter Set",  HITRAN#HT_DeleteParFolder()
			"Clean Up Auxiliary Data", HITRAN#HT_CleanupAuxData()			// remove all the Var_* and *_Coefs used for simulations.  
			"Load PAR File", HITRAN#HT_LoadParFileGUI()			
			"Save PAR Binary", HITRAN#HT_SaveParBinary()
		End
		"-"
			"Atmospheric Fractions",HITRAN#SA_MainPanel()
			"Molecule Color Codes", HITRAN#ColorCodePanel()
		"-"
			"Update Line Data", HT_CalcVarLineDataWrapper()
			"Plot Line Data", HITRAN#PlotLinesMenu()
			"Line Plot Legend", HITRAN#LinePlotLegendWrapper()
		"-"
			"Calculation Advisor", HITRAN#CA_CalculationAdvisorPanel()
			"Calculate Bands Accurate", HT_CalcBandsWrapper()
//		"Calculate Bands Hybrid"		// ?: bin widths into dominant ranges and add a few components.
			"Calculate Bands Fast", HT_CalcBandsOneWidthWrapper()
End

//
// SETUP FUNCTIONS
//

Static Structure HT_PrefsStruct
	char BinPath[256]
	char AscPath[256]			// Do I really need to save this?  Why not.  Easier to change my mind if I leave it in.
	char DefaultParFile[256]
	int16 LoadDefaultPar			// 1 for load "LastParFile" if it is found, 0 for always ask for an open dialog
EndStructure

//  CreatesOrUpdates the two HITRAN database file locations: Future version perhaps with a control panel instead of DoPrompt
Static Function HT_SetPreferences()
	Struct HT_prefsStruct HTprefs
	variable LoadDefaultPar
	string DefaultParFile
	String BinPath
	String AscPath
	LoadPackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	if (V_bytesRead)		// Yes there is a preferences file
		LoadDefaultPar = HTPrefs.LoadDefaultPar + 1
		BinPath = HTPrefs.BinPath
		AscPath = HTPrefs.AscPath
		DefaultParFile = HTPrefs.DefaultParFile
	else					// No Preferences Found
		HT_InitializePreferences()	
	endif
	prompt BinPath, "Default Path for HTIRAN binary package files"
	prompt AscPath, "Default Path for HITRAN Ascii data"
	prompt DefaultParFile, "Default Line Parameters File"
	prompt LoadDefaultPar, "Autoload Default Line Parameters Set?", popup, "no;yes"
	string HelpString = "Change Default Paths to have Igor start the search for package files in a new directory.\r\r"
	 HelpString += "If Autoload Default Parameters is set to yes, then the file specified by Default Parameters Binary will be loaded automatically in new package setups.\r"
	DoPrompt /HELP=HelpString "HITRAN Package Preferences", BinPath, AscPath, DefaultParFile, LoadDefaultPar
	if (V_flag)
		return 0
	else
		HTPrefs.LoadDefaultPar = LoadDefaultPar - 1
		HTPrefs.BinPath = ParseFilePath(2,ParseFilePath(5,BinPath,":",0,0),":",0,0)		// Convert to Mac style path and add trailing separator
		HTPrefs.AscPath =   ParseFilePath(2,ParseFilePath(5,AscPath,":",0,0),":",0,0)
		HTPrefs.DefaultParFile =  DefaultParFile

		SavePackagePreferences  "HITRAN", "HTPrefs.bin", 0, HTprefs
		return 1
	endif
End

Static Function HT_InitializePreferences()
	Struct HT_prefsStruct HTprefs
	string S_Docspath
	S_Docspath = SpecialDirPath("Documents",0,0,0)
	NewPath /O /Q HITRANBINPATH S_Docspath
	NewPath /O /Q HITRANASCPATH S_Docspath
	HTprefs.BinPath = S_Docspath
	HTprefs.AscPath = S_Docspath
	HTprefs.DefaultParFile = ""	// empty string, no par file.
	HTprefs.LoadDefaultPar = 0
	SavePackagePreferences "HITRAN", "HITRANHTprefs.bin", 0, HTprefs
End

// Creates datafolder tree and some of the globals used by the package.
Static Function CreateHITRANPackageGlobals()
	DFREF cdf = GetDataFolderDFR()
	newdatafolder /o /s root:Packages
	newdatafolder /o /s HITRAN
	string /g S_recentparfolder = ""
	variable /g recenttemp = STANDARDTEMP
	variable /g recentpress = STANDARDPRESS
	variable /g recentfreqlow = 0
	variable /g recentfreqhigh = Inf
	variable /g recentwnspacing = 1
	newdatafolder /o temp
	newdatafolder /o molparamdata
	newdatafolder /o parsumdata
	newdatafolder /o externaldata
	newdatafolder /o PARSets
	SetDataFolder cdf
End
	
// Initial setup to load text data downloaded from the HITRAN website.  
// Need files molparam.txt, parsum.dat and *.par (HITRAN08.par for the complete set, or use by molecule 
// files to build up a database one at a time)
// Default *.par file is HITRAN08.par, but if not found, user can select another *.par
// UPDATE changed my mind.  Make user select first *.par file.  If they have the full database and don't want to use it, they 
// should not have to.  Especially on a slow or memory limited system.

Static Function SetUpHITRANPackage()
	string cdf = GetDataFolder(1)
	STRUCT HT_PrefsStruct HTprefs
//	variable recordID
	LoadPackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	string S_ascpath, S_binpath
	CreateHITRANPackageGlobals()
	if (V_bytesRead)	// Package Preferences were found
		Print "Found HITRAN Package Preferences"
		S_ascpath = HTprefs.AscPath
		Print "Default Path For ASCII files is"
		Print S_ascpath
		NewPath /O /Q /Z HITRANASCPATH, S_ascpath
		if (V_flag)		// Oops, path from preference file no longer exists
			Print "Not a valid path, reverting to default value:"
			S_ascpath = SpecialDirPath("Documents",0,0,0)
			Print S_ascpath
			NewPath /O /Q /Z HITRANASCPATH, S_ascpath
		endif
		S_binpath = HTprefs.BinPath			
		Print "Default Path For Binary files is"
		Print S_binpath
		NewPath /O /Q /Z HITRANBINPATH, S_binpath
		if (V_flag)		// Oops, path from preference file no longer exists
			Print "Not a valid path, reverting to default value:"
			S_binpath = SpecialDirPath("Documents",0,0,0)
			Print S_binpath
			NewPath /O /Q /Z HITRANBINPATH, S_binpath
		endif
	else
		HT_InitializePreferences()
		Print "No Package Preferences Found"
		Print "Setting Start Paths to:"
		Print SpecialDirPath("Documents",0,0,0)
	endif
		
	Open /Z /D=2 /R /P = HITRANASCPATH /F="Text File:.txt;" /M =  "Please find molparam.txt" refnum as "molparam.txt"
	if (!StrLen(S_Filename))	// file not found and user cancelled dialog
		return 0
	endif
	setdatafolder root:Packages:HITRAN:molparamdata        		
	LoadMolParamf(filename = S_Filename)
	S_AscPath = ParseFilePath(1, S_FileName, ":", 1,0)	
	NewPath /O /Q HITRANASCPATH , S_AscPath		// redirect HITRANASCPATH if necessary
	wave moltope_lookuptable
	wave molnum
	wave /t molname
	make /o /t /n =  (wavemax(molnum) + 1) molnum_code		
	molnum_code = molname[moltope_lookuptable[p][1]]
	molnum_code[0] = ""
	
	PathInfo /S HITRANBINPATH // Should have a valid path in here by now so next NewPath dialog will start here.
	NewPath /M = "Select Folder to Save Package Binary Data" /O /Q /Z HITRANBINPATH		// Make user confirm default or change the first time.
	if (V_flag)	// User cancelled dialog
		return 0
	endif
	PathInfo HITRANBINPATH
	S_binpath = S_Path
	HTprefs.BinPath = S_binpath
	SaveData /O /P=HITRANBINPATH /Q "molparamdata.pxp"

	Open /Z /D=2 /R /P = HITRANASCPATH /F="Text File:.dat;" /M =  "Please find parsum.dat" refnum as "parsum.dat"
	if (!StrLen(S_Filename))	// file not found and user cancelled dialog
		return 0
	endif
	setdatafolder root:Packages:HITRAN:parsumdata     		
	LoadParSumData(filename = S_Filename)
	ReconcileParFileColumns()
	S_AscPath = ParseFilePath(1, S_FileName, ":", 1,0)	
	NewPath /O /Q HITRANASCPATH , S_AscPath		// redirect HITRANASCPATH if necessary
	SaveData /O /P=HITRANBINPATH /Q "parsumdata.pxp"		


	setdatafolder root:Packages:HITRAN:externaldata
	make /o /d /n = (wavemax(molnum) + 1) molfraction
	variable cnt
	for (cnt = 1; cnt < numpnts(molnum_code); cnt += 1)
		SetDimLabel 0,cnt,$molnum_code[cnt],molfraction
	endfor
	// Do I really want to hard code this into the function?  I could include as a binary wave, but whatever...
	// Data from Wikipedia: Atmosphere of Earth, middle of range used for variable quantities (i.e., water and ozone)
	molfraction[0] = 9.363e-3	// 	This is mostly Argon, which has no spectral lines in the HITRAN database
				//, but presence of Ar, He Ne etc. affects pressure broadening calculation
	molfraction[%H2O] = 0.02   // change to 0.0312632 for 100% hum at 25C
	molfraction[%N2] = 0.78084
	molfraction[%O2] = 0.20946
	molfraction[%CO2] = 3.8e-4
	molfraction[%CH4] = 1.79e-6
	molfraction[%N2O] = 3e-7
	molfraction[%O3] = 3.5e-8
	molfraction[%NO2] = 2e-8
	molfraction[%CO] = 1e-7
	duplicate /o molfraction, molfraction_standard
	SaveData /O /P=HITRANBINPATH /Q "externaldata.pxp"
	
	HTPrefs.AscPath = S_Ascpath
	HTPrefs.BinPath = S_Binpath
	SavePackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	
	SetDataFolder root:Packages:HITRAN:PARsets
	
//     Par File Section
	variable refnum
	Open /D /R /M = "Select a HITRAN line parameters (*.par) file" /R /P= HITRANASCPATH /F = "Parameters File (*.par):.par;All Files:.*;" refnum
	if (StrLen(S_FileName))
		string basename = ParseFilePath(3, S_fileName, ":", 0, 0) 
		newdatafolder /o /s $basename
		SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
		S_recentparfolder = basename
		LoadHitranPar(S_fileName)
		HTPrefs.DefaultParFile = basename + ".pxp"
		SaveData /O /P=HITRANBINPATH /Q (basename + ".pxp")
		SavePackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	endif
	setdatafolder cdf
	CreateCtabWave(CT_FOR_MOLECULES)
End

// creates Package folders and loads Binary data from saved packed .pxp files
// This should typically be run once per new Igor experiment after having run the initial setup procedure.
// If there is a valid Binaries Path specified in the HITRAN package preferences, it will look there first for each *.pxp file 
// and load without asking the user.  If it doesn't find one, it will query the user to find and reassign the "HITRANBINPATH"
//
//  Will also automatically load the default parameters IF the package preferences have been set to do so (by default it won't)
Static Function HT_InitializePackage()
	CreateHITRANPackageGlobals()
	string cdf = GetDataFolder(1)
	Struct HT_prefsStruct HTprefs
	LoadPackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	string S_binpath
	if (V_bytesRead)		// Yes there is a preferences file
		S_binpath = HTPrefs.BinPath
		Print "Default Path For Binary files is"
		Print S_binpath
		NewPath /O /Q /Z HITRANBINPATH, S_binpath
		if (V_flag)		// Oops, path from preference file no longer exists
			Print "Not a valid path, reverting to default value:"
			S_binpath = SpecialDirPath("Documents",0,0,0)
			Print S_binpath
			NewPath /O /Q /Z HITRANBINPATH, S_binpath
		endif
	else					// No Preferences Found
		HT_InitializePreferences()
		S_binpath = HTPrefs.BinPath
	endif
		
	variable refnum
	string pxpfilter = "Igor Packed Binary File:.pxp;"
	Open /Z /D=2 /R /P = HITRANBINPATH /F=pxpfilter /M = "Please find parsumdata.pxp" refnum as "parsumdata.pxp"
	if (!StrLen(S_Filename))	// file not found and user cancelled dialog...V_flag does not work as advertised in help
		print "cancelled"
		return 0
	endif
	setdatafolder root:Packages:HITRAN:parsumdata        	// partition matrix info
	NewPath /O /Q HITRANBINPATH , ParseFilePath(1, S_FileName, ":", 1,0)
	LoadData /Q /O = 1 S_fileName
	
	Open /Z /D=2 /R /P = HITRANBINPATH /F=pxpfilter /M =  "Please find molparamdata.pxp" refnum as "molparamdata.pxp" // Information about molecules and isotopes
	if (!StrLen(S_Filename))	// file not found and user cancelled dialog
		return 0
	endif
	setdatafolder root:Packages:HITRAN:molparamdata        		// partition matrix info
	NewPath /O /Q HITRANBINPATH , ParseFilePath(1, S_FileName, ":", 1,0)
	LoadData /Q /O = 1 S_fileName
	
	Open /Z /D=2 /R /P = HITRANBINPATH /F=pxpfilter /M =  "Please find externaldata.pxp" refnum as "externaldata.pxp" // Information about molecules and isotopes
	if (!StrLen(S_Filename))	
		return 0
	endif
	setdatafolder root:Packages:HITRAN:externaldata        		// partition matrix info
	S_binpath = ParseFilePath(1, S_FileName, ":", 1,0)
	NewPath /O /Q HITRANBINPATH , ParseFilePath(1, S_FileName, ":", 1,0)	
	LoadData /Q /O = 1 S_fileName
	HTprefs.binpath = S_binpath
	SavePackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs	// Save Prefs Here so If user cancels, will still be saved

	setdatafolder root:Packages:HITRAN:PARSets:
	// Now loading Par file data:
		
	S_FileName = HTprefs.DefaultParFile
	GetFileFolderInfo /Z /Q /P = HITRANBINPATH S_FileName		// Search for default file (set V_flag true if not found)
	if (!HTprefs.LoadDefaultPar)
		V_flag = 1												// Set V_flag true if pref is not to autoload.
	endif	
	if (V_flag)													// if default is not desired or does not exists
		Open /D /M = "Select compressed binary line parameters file (*.pxp)" /R /P= HITRANBINPATH /F = "Compressed Binary File (*.pxp):.pxp;All Files:.*;" refnum
		if (!strlen(S_fileName))		// cancelled
			return 0
		endif
		HTprefs.DefaultParFile =  ParseFilePath(3, S_fileName, ":", 0, 0)  + ".pxp"
	endif														// Either way S_Filename should be correct now.
	string basename = ParseFilePath(3, S_fileName, ":", 0, 0)

	if (datafolderexists(basename))
		basename = UniqueName(basename, 11, 0)
	endif
	newdatafolder /o /s $basename
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	S_recentparfolder = basename
	LoadData /P = HITRANBINPATH /O = 1 /Q S_fileName	
	setdatafolder cdf
	CreateCtabWave(CT_FOR_MOLECULES)
	SavePackagePreferences  "HITRAN", "HTPrefs.bin", 0, HTprefs
End

// Mixes up a standard color table (What I really need is a ColorTable that is purposely high contrast between adjascent indices.)
// ColorTableName is allowed as an option to potentially allow user selection in the future, but for now SpectrumBlack is locked in.  
Static Function CreateCtabWave(ColorTableName)
//	variable NumMolecules
	string ColorTableName
	DFREF cdf = GetDataFolderDFR()
	setdatafolder root:Packages:HITRAN
	string /g CTformols = ColorTableName
	wave /t molnames = root:Packages:HITRAN:molparamdata:molnum_code
	variable nummols = numpnts(molnames)
	make /o /u /i /n = (nummols,3) HT_ctabmol		// RBG wave indexed by molecule number
	ColorTab2Wave $CTformols
	wave M_colors
//	HT_ctabmol = M_colors[p*DimSize(M_colors,0) / nummols][q]
	HT_ctabmol = M_colors[mod(p*43,DimSize(M_colors,0))][q]		// need a better way to mix these up.  Works nicely for now with 43 molecule types
	killwaves /z M_colors
	setdatafolder cdf
End


static Function HITRANAutoSetup()
	Struct HT_prefsStruct HTprefs
	if (WaveExists($(HT_DF + "molparamdata:moltope_lookuptable")))	// package globals already present in this experiment file
		return 1
	endif
	LoadPackagePreferences "HITRAN", "HTPrefs.bin", 0, HTprefs
	string S_binpath
	Variable StartupMode
	if (V_bytesRead)		// Yes there is a preferences file
		StartupMode = 2		// call Load Binaries
	else
		StartupMode = 1		//  First time setup...convert text files
	Endif
	Prompt StartupMode, "Mode", popup, "First Run (Convert HITRAN text files);Load HITRAN Binary Data;Do Nothing"
	string Helpstring = "Select \"First Run\" to convert HITRAN database text files to Igor binaries. \r"
	HelpString += "Select \"Load Binaries\" if you have already run the initial setup and retained the package binaries. \r"
	HelpString += "Select \"Do Nothing\" or \"Cancel\" if you do not wish to set up the package structures in the current experiment file. "
	DoPrompt /HELP = HelpString "HITRAN Package Initialization:", StartupMode
	if (V_flag)
		return 1
	endif
	switch (StartupMode)
		case 1:
			SetUpHITRANPackage()
			break
		case 2:
			HT_InitializePackage()
			break
		default:
	endswitch
	return 1
End

//  No... I think I need to do this with a prompt and a popup
//static Function HITRANAutoSetupPanel(FoundPrefs)
//	variable FoundPrefs
//	NewPanel /N = HT_AutoSetupPanel // /FLT=1  // enable /FLT = 1 eventually
//	SetDrawLayer UserBack
//	if (FoundPrefs)
//		DrawText 22,33,"Preferences File Found"
//	else
//		DrawText 22,33,"No Preferences File Found"
//	endif
//	Button button_okay,pos={20,140},size={50,20},title="Okay"
//	Button button_Cancel,pos={80,140},size={50,20},title="Cancel"
//	GroupBox group_options,pos={16,50},size={133,60},title="Startup Mode"
//	CheckBox check_Init,pos={24,70},size={117,14},title="Run First Time Setup"
//	CheckBox check_Init,value= (!FoundPrefs),mode=1, proc = HT_AS_checkbox
//	CheckBox check_Start,pos={24,88},size={82,14},title="Load Binaries"
//	CheckBox check_Start,value= (FoundPrefs),mode=1, proc = HT_AS_checkbox
//	
//End
//
//Function HT_AS_checkbox(cba) : CheckBoxControl
//	STRUCT WMCheckboxAction &cba
//	switch( cba.eventCode )
//		case 2: // mouse up  but note event not triggered if it was already checked (must programmatically uncheck radio buttons)
//			Variable checked = cba.checked
//				strswitch (cba.CtrlName)
//					case "check_Init":
//						CheckBox check_Start, value = 0
//						break
//					case "check_Start":
//						CheckBox check_Init, value = 0
//						break
//				endswitch
//			break
//		case -1: // control being killed
//			break
//	endswitch
//
//	return 0
//End
//
//Function HT_AS_ButtonProc(ba) : ButtonControl
//	STRUCT WMButtonAction &ba
//
//	switch( ba.eventCode )
//		case 2: // mouse up
//			// click code here
//			break
//		case -1: // control being killed
//			break
//	endswitch
//
//	return 0
//End


//  END SETUP FUNCTIONS


// LOAD PROCEDURES

// HITRAN Par File contains the line-by-line information for HITRAN *.par files
// Use with either the entire collection (a roughly 400 MB text file) or the by molecule files.
// Written for the 2004 format.  This will not work with older databases.
//
// Also loads HITRAN08 par files (the only format change is gamma_self went from code
// F5.4 to F5.3 which apparently affects writing text files, not reading them?.)
//
// Loads all the information in the *.par file into automatically named waves (I recommend only running
// in a fresh directory or if you know you intend to overwrite)
// 
//  Also possibly creates one new wave called "moltope" which is an index uniquely identifying the molecule and isotope
//  and which is widely used in the simulations (moltope is used to look up the correct partition function value as a function
//  of temperature...it is the column index of partmat and and rel_partmat in the "parsumdata" folder in the HITRAN projects
//  tree.  moltope can ONLY be created if  root:Packages:HITRAN:molparamdata:moltope_lookuptable  exists (which is
//  created by the setup functions.
//
//  This function can be called in isolation without setting up the whole HITRAN package, but will not create the moltope
//  wave.  If you later want to use this loaded *.par data in a simulation, you'll have to manually create moltope (follow the 
//  steps in the last five lines of this function.

Function LoadHITRANPar(filenamestr)
	string filenamestr
	string cinfstr = "C=1,F=0,T=72,W=2,N=molnum;C=1,F=0,T=72,W=1,N=isotope;"
	cinfstr += "C=1,F=0,T=4,W=12,N=freq;C=1,F=0,T=4,W=10,N=intensity;"
	cinfstr += "C=1,F=0,T=4,W=10,N=EinstA;C=1,F=0,T=2,W=5,N=gamma_air;"
	cinfstr += "C=1,F=0,T=2,W=5,N=gamma_self;C=1,F=0,T=2,W=10,N=EngGS;"
	cinfstr += "C=1,F=0,T=2,W=4,N=n_air;C=1,F=0,T=4,W=8,N=delta_air;"
	cinfstr += "C=1,F=-2,W=15,N=Vpr;C=1,F=-2,W=15,N=Vdblpr;"
	cinfstr += "C=1,F=-2,W=15,N=Qpr;C=1,F=-2,W=15,N=Qdblpr;"
	cinfstr += "C=1,F=0,W=19,N='_skip_';"
	cinfstr += "C=1,F=0,T=2,W=7,N=gprime;C=1,F=0,T=2,W=7,N=gdblprime;"
	LoadWave /O /Q /N /F={17,2,0} /P=home /B = cinfstr filenamestr
//	 LoadWave /O /Q /N /F={17,2,0} /P=home /V={"", "", 0, 0 } /B = cinfstr filenamestr 
	
	
	if (!V_flag)		// cancelled
		return 0
	endif
	wave EinstA
	EinstA = (NumType(EinstA) == 2) ? 0 : EinstA		// AFAIK, only EinstA has missing numbers
	// I've replaced them with 0 so I can still do emission calculations without NaNs messing everything up.
//	SetWaveLock 1, allinCDF
	wave /z moltope_lookup = root:Packages:HITRAN:molparamdata:moltope_lookuptable
	if (waveexists(moltope_lookup))
		wave molnum, isotope
		make /o /u /b /n = (numpnts(molnum)) moltope
		moltope = moltope_lookup[molnum][isotope]
	endif
End

// Opens a file dialog to select either a text based *.par file or an igor compressed binary *.pxp file and 
// loads the waves into an appropriately named package data folder.
//
// Calls the utility function "LoadHitranPar" if an ascii *.par file is selected, or just uses LoadData for a *.pxp

Static Function HT_LoadParFileGUI()
	string cdf = GetDataFolder(1)
	if (datafolderexists(HT_PARDF))
		setdatafolder $HT_PARDF
	endif
	string filefilter = "ASCII Parameters File (*.par) or Compressed Binary File (*.pxp):.par,.pxp;"
	variable refnum
//	Open /D /M = "Select a line parameters file" /R /P= HITRANASCPATH /F = filefilter refnum 
	Open /D /M = "Select a line parameters file" /R /F = filefilter refnum 
	if (StrLen(S_fileName))
		string ext = parsefilepath(4,S_fileName,":",0,0)
		string basename = parsefilepath(3,S_fileName,":",0,0)
		if (datafolderexists(basename))
			basename = UniqueName(basename, 11, 1)
		endif
		
		newdatafolder /o /s $basename
		if (stringmatch(ext,"pxp"))
			LoadData /O = 1 /Q S_fileName
		else
			LoadHitranPar(S_fileName)
		endif
		
		SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
		S_recentparfolder = QuoteLiberalName(basename)
	endif
	setdatafolder cdf
end

//  Save contents of a parameters set into a packed binary file (Basically just a "SaveData" wrapper.  
Static Function HT_SaveParBinary()
	string ParFolder
	prompt parfolder, "Parameters Folder", popup, ParFolderList()
	DoPrompt "Select Par Folder", parfolder
	if (V_Flag == 1) 
		return 0
	endif
	variable refnum
	Open /D /F= "Compressed Binary File (*.pxp):.pxp;All Files:.*;" /M = "Save Par Folder To" /P = HITRANBINPATH refnum as UnquoteLiberalName(parfolder)
	if (strLen(S_FileName))
		string cdf = GetDataFolder(1)
		setdatafolder $(HT_PARDF + parfolder)
		SaveData /O S_FileName
		setdatafolder cdf
	endif
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	S_recentparfolder = parfolder
End

// Kind of redundant since this can be easily done from the data browser, but easy enough to include for users not familiar with the data browser.
Static Function HT_DeleteParFolder()
	string ParFolder
	prompt parfolder, "Parameters Folder (Are you sure!?!?)", popup, ParFolderList()
	DoPrompt "Delete Par Folder", parfolder
	if (V_Flag == 1) 
		return 0
	endif
//	parfolder = ParFolderNameParser(parfolder)
	KillDataFolder $(HT_PARDF + parfolder)
End
	
// call optional as LoadMolParamf(filename="molparam.txt")
// Use this for "molparam.txt" from the Hitran downloads.
// I hope they don't mess with this format too much in future versions!!!  what a pain it was to parse.  
//
// This data essentially sets the indexing (the relationship between molecule number, moltope number and chemical species
// in addition to conaining necessary info for simulations.  (Actually, I don't think I ever use the molarmass or the degeneracy, but
// they're parsed with the rest of the column, so leave them in for reference.
//
// Quick explanation of loaded waves:
// molarmass is indexed by moltope, carries molar mass in amu
// Q_RT indexed by moltope, It's the value of the partition function at room temperature
// Abundance indexed by moltop, holds isotopic abundance by chemical species of each isotope (i.e. should
//	 sum to 1 over every group of points corresponding to the same molecule)
// isotopenum is the shorthand description of each isotope (each atomic isomer reduced to least significant
// digit i.e. H2O has 181 (1H, 18O, 1H), 271 (2D, 17O, 1H)
// isotopecode is the 1-based index of which isotope, reset for each molecular species
// molnum is the 1-based index of molecular species, indexed over moltope
// molname is the chemical formula of each molecule, indexed over moltope
Function LoadMolParamf([filename])
	string filename
	variable fileref
	if (ParamIsDefault(filename))
		open /D /R fileref
		if (strlen(S_filename))
			open /R fileref as S_filename
		endif
	else
		open /Z /R fileref as filename
	endif
	if (V_flag != 0)
		return 0
	endif
	
	string loadline = ""
	FReadLine fileref, loadline		// load header titles, for now I won't use
	variable V_molnum = 0, linecnt = 0, itemcnt = 0
	make /o /d /n = (300) Abundance, Q_RT, molarmass
	make /o /i /n = (300) Isotopenum
	make /o /b /n = 300 gj, molnum, isotopecode
	make /o /t /n = 300 molname
	variable V_Isotopenum, V_abundance, V_Q_RT, V_gj, V_molarmass, V_isotopecode
	string S_molname = ""
	do
		FReadLine fileref, loadline
		linecnt += 1
		if (!strlen(loadline))
			break
		endif
//		sscanf loadline, " %s (%d)", S_molname, V_molnum
//		if (V_flag == 2)
//			continue
//		endif
		sscanf loadline, " %d %f %f %d %f", V_Isotopenum, V_abundance, V_Q_RT, V_gj, V_molarmass
		switch (V_flag)
			case 0:		// Not an info line, should be a label line (molecule name and numcode)
				sscanf loadline, " %s (%d)", S_molname, V_molnum
//				V_molnum += 1
				V_isotopecode = 1
				break
			case 5:
				molnum[itemcnt] = V_molnum
				Isotopenum[itemcnt] = V_Isotopenum
				Abundance[itemcnt] = V_Abundance
				Q_RT[itemcnt] = V_Q_RT
				gj[itemcnt] = V_gj
				molarmass[itemcnt] = V_molarmass
				molname[itemcnt] = S_molname
				isotopecode[itemcnt] = V_isotopecode
				itemcnt += 1
				V_isotopecode += 1
				break
			default:
				print "error, mismatched line at ", linecnt
				return 0
		endswitch
		
	while (1)
	close fileref
	
	redimension /n = (itemcnt)  Abundance, Q_RT, molarmass, Isotopenum, gj, molnum, molname, isotopecode
	wavestats /q molnum
	variable maxmol = V_max
	wavestats /q isotopecode
	variable maxtope = V_max
	make /o /b /n = (maxmol + 1,maxtope + 1) moltope_lookuptable
	moltope_lookuptable = -1 	// will fill all used entries with 0 or a positive number, -1 marks an invalid lookup.
	for (itemcnt = 0;itemcnt < numpnts(molnum);itemcnt += 1)
		moltope_lookuptable[molnum[itemcnt]][isotopecode[itemcnt]] = itemcnt
	endfor
	make /o /t /n = (itemcnt) moltopename
	moltopename = molname + "_" + num2str(isotopenum)
	return 1
End

//NOTE: The columns don't quite match up to the list of molecules and isotopes in the HITRAN.par and 
// molarparam.txt files.  A few extra and a few missing.   See the ReconcileParFileColumns Function.  
Function LoadParSumData([filename])
	string filename
	variable fileref
	if (ParamIsDefault(filename))
		open /D /R fileref
		if (strlen(S_filename))
			filename = S_filename
		else
			return 0
		endif
	endif
		
//	LoadWave /D /J /M /U={0, 2, 1, 2} /V={" ", "", 4,0}  filename
//	LoadWave /D /M filename

	LoadWave /Q /W /A /D /O /G filename
	
	variable cnt
	wave temperature = $stringfromlist(0, S_waveNames)
	make /o /d /n = (numpnts(temperature),V_Flag - 1) partition_mat
	make /o /t /n = (V_Flag - 1) columnnames
	
	for (cnt = 1; cnt < V_Flag; cnt += 1)
		wave thiscol = $stringfromlist(cnt, S_waveNames)
		partition_mat[][cnt - 1] = thiscol[p]
		SetDimLabel 1,(cnt - 1),$stringfromlist(cnt, S_wavenames),partition_mat
		columnnames[cnt - 1] = stringfromlist(cnt, S_wavenames)
		killwaves thiscol		// Do I want these? comment out if so.
	endfor
	
	setscale /p x, temperature[0], (temperature[1] - temperature[0]), "" partition_mat
	
//	duplicate /o partition_mat relpart_mat
//	relpart_mat = partition_mat / partition_mat(296)[q]		// relative partition matrix to 296, since partition matrix
	// is already included in the intensity column.
		// Don't need relpart_mat anymore after changing approach to line calculations.  Kind of a waste of space.  	
end	



// Older HITRAN format.  
// Read HITRAN documentation (I think the '96 paper) for explanation of the 100 character format.  

Function LoadHitran100Filef(keepxtras, filenm)	// 100 character format.
	variable keepxtras
	string filenm
	variable fileref
	if (strlen(filenm))	// if a file name was sent
		Open /Z = 2 /R fileref as filenm
	else					// else query the filename with a dialog.
		if (stringmatch(IgorInfo(2), "Macintosh"))
			Open /R fileref
		else
			Open /R  /T=".out" fileref
		endif
	endif
	if (!fileref)
		print "Cancelled"
		return NaN
	endif


	string shortname = S_filename[strsearch(S_filename, ":", Inf,1) + 1, strlen(S_filename) - 5]
//	string shortname = S_filename[StrSearchBack(S_filename, ":") + 1, strlen(S_filename) - 5]
	shortname = CleanupName(shortname, 0)

	variable owflag = 1
	prompt shortname, "BaseName"
	prompt owflag, "Should I...", popup, "Use a new basename;Overwrite previous data"
	Do
		If (waveexists($("Freq_" + shortname)))
			DoPrompt "This BaseName is already in use.", shortname, owflag
			If (V_flag)
				print "cancelled"
				return 0
			endif
		else
			owflag = 2
		endif
	While (owflag == 1)
	
	string cinfstr
	if (keepxtras == 2)	// keep all waves
		cinfstr = "N=mol_" + shortname + ",T=8,W=2;N=iso_" + shortname + ",T=8,W=2;N=Freq_" + shortname + ",T=4,W=12;"
	else					// just load frequency, intensity and linestrength
		cinfstr = "N='_skip_',W=3;N=Freq_" + shortname + ",T=4,W=12;N=Inten_" + shortname + ",T=4,W=10"
	
	endif
	
// 	not done.
	
End		// loadHitran100filef()


// Fixes the columns in the partition matrix (partition_mat) to match the moltope indexing established in molparam.txt
// Cannot run until both parsum.dat and molparam.txt have been loaded.  As of the HITRAN08 database
// this required stripping some extra columns (O3 isotopomers) and adding some missing columns.  Added columns are just
// filled with a constant value (= to room temp partition function which was supplied in molparam.txt) and can be used in isolation
// for simulations if relative band structure is all one cares about (absolute absorptions / emissions will be completely off).  
//  Also important to note these are isotopically very rare species (except NO+) so they will screw up composition relative to 
//  the other isotopomers, but are probably incredibly weak lines to begin with.  
//
// Identities of removed and added species are stored in string variable lists.  As of this version, they were
//  AddedColumns = "CO2_838;CO2_837;CH4_312;NO+_46;CH3Br_219;CH3Br_211;"
//  RemovedColumns = "O3_678;O3_767;O3_768;O3_776;O3_777;O3_778;O3_786;O3_787;O3_868;O3_878;O3_886;O3_887;O3_888;NO__46;"
Static Function ReconcileParFileColumns()
	wave parmat = root:Packages:HITRAN:parsumdata:partition_mat
	wave /t columnnames= root:Packages:HITRAN:parsumdata:columnnames
//	wave relpart_mat = root:Packages:HITRAN:parsumdata:relpart_mat
	wave /t moltopename = root:Packages:HITRAN:molparamdata:moltopename
	wave Q_RT = root:Packages:HITRAN:molparamdata:Q_RT
	
	duplicate /o parmat,root:Packages:HITRAN:parsumdata:partition_mat_orig
	duplicate /o columnnames, root:Packages:HITRAN:parsumdata:columnnames_orig
	variable cnt = 0
	string cdf = GetDataFolder(1)
	setdatafolder  root:Packages:HITRAN:parsumdata		// I should only call from within Parsumdata anyway, but just to be sure.
	String /g AddedColumns = ""
	String /g RemovedColumns = ""
	// first strip partition matrix columns that aren't matched in molecular parameter data
	for (cnt = 0; cnt < numpnts(columnnames); cnt += 1)
		if (FindInTextWave(moltopename,columnnames[cnt]) == -1)
			RemovedColumns = AddListItem(columnnames[cnt], RemovedColumns, ";", Inf)
			DeletePoints /M=0  cnt, 1, columnnames
			DeletePoints /M=1  cnt, 1, parmat
			cnt -= 1 
		endif
	endfor
	// now insert missing moletope rows
	for (cnt = 0; cnt < numpnts(moltopename); cnt += 1)
		if (!stringmatch(columnnames[cnt],moltopename[cnt]))
			AddedColumns = AddListItem(moltopename[cnt],AddedColumns, ";", Inf)
			InsertPoints /M=0  cnt, 1, columnnames
			InsertPoints /M=1  cnt, 1, parmat
			columnnames[cnt] = moltopename[cnt]
			parmat[][cnt] = Q_RT[cnt]
			SetDimLabel 1,(cnt),$columnnames[cnt],parmat
//			cnt += 1 // can skip the recently inserted data, I know it will be there
		endif
	endfor
//	duplicate /o parmat relpart_mat
//	relpart_mat = parmat / parmat(296)[q]
	SetDataFolder cdf		
End

// finds first occurance of matchstr (wildcards * allowed) in the textwave
Static Function FindInTextWave(twave, matchstr)
	wave /t twave
	string matchstr
	variable cnt
	for(cnt = 0; cnt < numpnts(twave); cnt += 1)
		if (stringmatch(twave[cnt],matchstr))
			return cnt
		endif
	endfor
	return -1
End



// END LOAD PROCEDURES



// PARAMETER SET MANIPULATIONS

// Removes "extra" waves created during simulations (I don't normally delete them because they get reused so often)
// Note VarInten and VarFreq are often plotted on a stick plot which will keep them from being deleted.  /Z flag in killwaves means
// the delete will fail silently if they are in use.  

Static Function HT_CleanupAuxData()
	string ParFolder
	prompt parfolder, "Parameters Folder", popup, ParFolderList()
	DoPrompt "Remove Auxiliary Data", parfolder
	if (V_Flag == 1) 
		return 0
	endif
	string cdf = GetDataFolder(1)
	setdatafolder root:Packages:HITRAN:PARSets:$parfolder
	killwaves /Z VarInten, VarFreq, VarEmit, LorWidth, GaussWidth, VoigtWidth, LorWidth_Hist, GaussWidth_Hist
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	S_recentparfolder = parfolder	
	setdatafolder cdf

End



// This is a simple combine that does not check for duplicates.  It would be nice to, but given the potential for an update to change any of the fields, and the lack of a unique
// lineID in HITRAN, I do not know if it can be done unambiguously.  So if the same data are included in both folders they will become two different lines be overcounted in 
// cross section calculations.  
//  For now, this will be useful for including by-molecule updates, if you first just strip out the previous members.  AFAIK 
// that's how HITRAN issues minor updates: a new h2o.par file should have all water lines, even if only a few have been changed or added.  
//  To update your entire HITRAN database to include the new h2o data, it is first necessary to remove the old h2o data (make a subset with all other species)
// then load the new h2o.par file and combine it back in.  
//
Static Function CombineParFolders(SrcFolder,DestFolder)
	string SrcFolder, DestFolder
//	SrcFolder = ParFolderNameParser(SrcFolder)
//	DestFolder = ParFolderNameParser(DestFolder)
	
	string SrcFullPath = "root:Packages:HITRAN:PARSets:" + SrcFolder
	string DestFullPath = "root:Packages:HITRAN:PARSets:" + DestFolder
	
	if (!DataFolderExists(SrcFullPath))
		print "Could Not Find ", SrcFolder
		return 0
	endif
	if (!DataFolderExists(DestFullPath))
		print "Could Not Find ", DestFolder
		return 0
	endif
	string cdf = GetDataFolder(1)
	SetDataFolder root:Packages:HITRAN:PARSets:$SrcFolder
	string SrcWavelist = WaveList("*",";","")
	wave freq = freq
	variable SrcRows = NumPnts(freq)
	SetDataFolder cdf
	SetDataFolder root:Packages:HITRAN:PARSets:$DestFolder
	wave freq = freq
	variable DestRows = NumPnts(freq)
	variable NewRows = SrcRows + DestRows
	variable cnt
	for (cnt = 0; cnt < itemsinlist(SrcWavelist); cnt += 1)
		SetDataFolder SrcFullPath
		wave SrcWave = $StringFromList(cnt,SrcWaveList)
		SetDataFolder DestFullPath
		wave /z DestWave = $StringFromList(cnt,SrcWaveList)
		string SrcWaveFullName = GetWavesDataFolder(SrcWave,2) + ";"
		if (waveexists(DestWave))
//			redimension /n = (NewRows) DestWave
//			DestWave[DestRows,] = SrcWave[p - DestRows]
			concatenate /NP SrcWaveFullName, DestWave
		endif
	endfor
	setDataFolder cdf
	SortParFolder(DestFolder)		
end

//  Wrapper function for CombineParFolders()
Static Function HT_CombineParFoldersMenu()
	string ParSrc, ParDest
	Prompt ParSrc, "Source Par Folder", popup, ParFolderList()
	Prompt ParDest, "Destination Par Folder", popup, ParFolderList()
	DoPrompt "Select Parameter Folders", ParSrc, ParDest
	If (V_Flag)
		return 0
	endif
	if (stringmatch(ParSrc, ParDest))
		print "Cannot Copy Source Into Itself"
		return 0
	endif
	CombineParFolders(ParSrc,ParDest)
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	S_recentparfolder = ParDest
End

//  Not sure I want to enable user -resorting.  Generally code assumes parameter sets are sorted by frequency.  Needed
//  to resort by frequency after combining two sets.  I have occasionally found it advantageous to sort lines by other parameters
//  for purposes of spectral identification, but for now I will leave that as requiring a manual call to this function from the command
//  line.  Be wary of leaving parameter folder out of frequency order.  
//
// Sorts all files in a parameters folder with the same length as the number of lines 
// (Auxiliary and user created waves will not screw up the sort command)
// 
// SortBy is a string containing either a wave name (just the name, not a full path) 
// or a bracket enclosed list of wave names.  It will be passed directly to MakeIndex as a string in an Execute command,
// so it can be the name of a single wave inside ParFolder, or it can be a bracket enclosed list of 
// wave names (full path to waves not necessary, the execute will be run from inside the target ParFolder).
//
// Default is to sort by the "freq" wave only
// 
//  E.G. SortParFolder("NH3Pars","{freq,EngGs}")
//  Sorts by frequency first and then by ground state energy second.
Static Function SortParFolder(ParFolder,[SortBy])
	string ParFolder, SortBy
	if (ParamIsDefault(SortBy))
		SortBy = "freq"
	endif
	string cdf = GetDataFolder(1)
	setdatafolder root:Packages:HITRAN:PARSets:
	if (!DataFolderExists(ParFolder))
		setdatafolder cdf
		return 0
	endif
	SetDataFolder ParFolder
	wave freq			// there should always be a freq wave here.
	string nrstr = ""
	sprintf nrstr, "%d", numpnts(freq)
	string TheWaves = wavelist("*",";",("MINROWS:" + nrstr + ",MAXROWS:" + nrstr))
	make /o /u /i /n = (numpnts(freq)) S_idx
	Execute ("MakeIndex 	" + SortBy + " S_idx")		// Use Execute, because I don't know how to build a list of waves for multiple key waves in function form
	variable cnt
	for (cnt = 0; cnt < itemsinlist(TheWaves); cnt += 1)
		wave thiswave = $stringfromlist(cnt,TheWaves)
		IndexSort S_idx, thiswave
	endfor
	killwaves S_idx
	SetDataFolder cdf
	return 1
End


// A pre-scan function that will run through the (presumably sorted by frequency) waves and figure out
// how many lines will ultimately be needed to make subset waves.  Called exclusively by MakeSubParFolder()
// but separated out just for functional clarity. 
//
// pass an empty or non-existant wave to moletop_include to include all molecules and all isotopes
// otherwise, it's a wave containg the moletope numbers that should be included.

// Fills idxout with the point numbers of qualifying lines for later use in making a data subset.

// matchwave and match_include were originally written for moletope, but I realized matching
// a molecule would be useful too.  So match_include is a list of numbers that have to be matched
// by the contents of matchwave.  If empty or non-existant, all numbers are valid and the function just counts line based on included frequencies
// 
// IE to count all NH3 lines use match_include = {11}, matchwave = root:Packages:HITRAN:pardata:molnum
// To count just the prominent isotope, use match_include = {42}, matchwave = root:Packages:HITRAN:pardata:moltope
Static Function CountLines(freqlow, freqhigh, intensitymin, match_include, freqwave, intenwave, matchwave, idxout)
	variable freqhigh, freqlow, intensitymin
	wave /z match_include 
	wave freqwave, intenwave, matchwave, idxout
	variable startpnt = BinarySearch(freqwave, freqlow) + 1
	variable endpnt = BinarySearch(freqwave, freqhigh)
	if (endpnt == -2)
		endpnt = numpnts(freqwave) - 1
	endif
	variable cnt, nummatches
	make /b /free /n = (endpnt - startpnt + 1) matched
	matched = 0
	if (waveexists(match_include))
		for (cnt = 0; cnt < numpnts(match_include); cnt += 1)	// loop through list of included criteria
			matched = matched || (match_include[cnt] == matchwave[p + startpnt])	// if any match, set line to true
		endfor
		matched = matched && (intenwave >= intensitymin)	// Now Apply intensity cutoff.  
		nummatches = sum(matched)
		redimension /n = (nummatches) idxout
		variable idxcnt = 0
		for (cnt = 0; cnt < (endpnt - startpnt + 1) ; cnt += 1)	// now loop through lines and mark idxout with matching lines
			if (matched[cnt])
				idxout[idxcnt] = cnt + startpnt
				idxcnt += 1
			endif
		endfor
		return nummatches
	else								// assumption is if I pass an empty reference for match_include, I want ALL lines in freq range, not NONE.  
// Intensity Cutoff NOT incorporated yet in the else block, but can wait.  I almost never call manually anymore, and GUI always supplies match_include
		redimension /n = (endpnt - startpnt + 1) idxout
		idxout = startpnt + p
		return (endpnt - startpnt + 1)
	endif	
	
End


// Call this function to pull out a subset of the line parameter data from a larger set.  Generally would want to run once on the whole
// database to pull out a small wavelength range or a small number of molecules, but it's also possible to call this on an already incomplete folder
// but, of course, it will not recover the missing data that had already been skipped.
// 
// matchwave is a numeric wave from the original par folder with line-by-line information used to select a match criterion.
// Typically will be molnum (number code of the molecule) to select a molecule or set of molecules, or moltope to select a
// single isotope of a single molecule. 
//
// Matchlist is a wave containing matchable numeric values.  
//
//  I.E. to make a subset of just water and CO2, matchwave = molnum, matchlist = {1,2}
//
//  I generally only use this from the Select Species GUI now and only to match to molecules or moletopes, but if you parse
//  the quantum numbers in Vpr, Vdblpr, Qpr and Qdblpr it can be a useful way to break up spectra
//

Static Function MakeSubParFolder(freqlow, freqhigh, intensitymin, matchlist, matchwave, parfolder,subparfolder)
	variable freqlow, freqhigh, intensitymin
	wave /z matchlist
	wave matchwave
	string parfolder
	string subparfolder
	string par_fullpath = HT_PARDF + parfolder
	string sub_fullpath = HT_PARDF + subparfolder
	string cdf = getdatafolder(1)
	NewDataFolder /s /o $sub_fullpath
	make /o /i /n = 0 origidx
	setdatafolder $par_fullpath
	wave freqwave = freq
	wave intensity

	// Only same length waves will be considered.  
	//  Auxiliary waves with other lenghts (like the fit function coefficient waves) will not be included in the copy
	
	string MatchOptions = ""
	sprintf MatchOptions, "MINROWS:%d,MAXROWS:%d,", numpnts(freqwave), numpnts(freqwave)
	
	string parwaves = Wavelist("!origidx",";",MatchOptions)
	variable subsize = CountLines(freqlow, freqhigh, intensitymin, matchlist, freqwave, intensity, matchwave,origidx)
	print "Found ", subsize, " matching lines"
	if (subsize == 0)
		DoAlert 1, "No Matching Lines Found.  Are you sure you want to create the subdirectory?"
		if (V_flag == 2)
			setdatafolder cdf
			killdatafolder $sub_fullpath	// Do I really want to do this?  At this point I could end up erasing
			// I think so, because even if I don't delete, I have erased the previous origidx wave, which I 
			// might use in the future.
			return 0
		endif
	endif
	variable cnt

// Two completely different methods of dealing with text waves follow.  The first is the original, straightforward method 
// that uses the same method as for a numeric: make a text wave with the correct number of elements, then fill one element at a time with a wave assignment statement
//
// The problem is, unlike numeric waves where each element is a guaranteed fixed length, assigning to an element of a text wave requires an unknown (until assignment) 
// amount of memory so the pre-made textwave may not have the correct memory size.  Even worse, Igor Textwaves are long continuous blocks of all characters from 
// all elements with an additional record of where each element break is, rather than a collection of pointers to dynamically allocated strings.  
// Because strings can be adjustable length, a single element assignment in a text wave requires a memory manipulation of the entire text wave (including a very 
// likely complete memory copy of all the data if larger block allocation was required), and the straightforward assignment can be brutally slow.
//
// The better solution would be to use fixed length fields for the text data that is loaded in the quantum "number" fields 
// (this problem does not arise if a new assignment to a text wave element uses the 
// same length string as the previous contents of the element, as Igor just overwrites the old characters without needing a new memory allocation).
//
// After I brought this issue up with Wavemetrics, Larry kindly added the optional =num argument to the /T flag in "make" so that a 
// new text wave can be generated with preallocated string lengths for each element.  However, the problem really 
// originates in loading text data from a *.par file because LoadWave strips whitespace from text columns (even in Fixed Width form like *.par files which should very easily
// produce common sized strings for text columns), and they have 
// not offered an option to suppress that behavior.  This produces non-uniform length strings in the quantum number information even within a single molecule.  
// I should update the LoadParFile to copy the text fields into a new uniform string length waves, but have not gotten to it yet.  
//
//  Also of course, my kludgey sort and delete method allows a user to have created their own text wave based information (like text based line information) 
// that may not be guaranteed to have a single string length, so I will probably just leave it.  The code is confusing, but not particularly inefficient.  
//
#ifdef ORIG

 // original way of doing it with potentially slow textwave assigments.  Retained for debugging purposes
 	print "orig method"
	for (cnt = 0; cnt < itemsinlist(parwaves); cnt += 1)
//		print ("Copying Lines From " + stringfromlist(cnt,parwaves))
		wave thiswave = $stringfromlist(cnt,parwaves)
		wave /t thiswavetext = $stringfromlist(cnt,parwaves)
		string newwavename = sub_fullpath + ":" + stringfromlist(cnt,parwaves)
		make /o /n = (subsize) /Y=(WaveType(thiswave)) $newwavename
		if (WaveType(thiswave))	// if thiswave is numeric
			wave destwave = $newwavename
			destwave = thiswave[origidx[p]]
		else						// no it's a text wave
			
			wave /t destwavetext = $newwavename
			destwavetext = thiswavetext[origidx[p]]		
		endif
	endfor
#else
//	print "alt method"
// 	alternate way: so far better.  Duplicate entire text wave, then shove all elements to be removed to the end, then delete the points.
	wave firstpar = $stringfromlist(0,parwaves)
	variable parsize = numpnts(firstpar)
	make /o /i /n = (parsize) forsort, sortidx
	forsort = 2^31	// all unassigned elements will be large
	for (cnt = 0; cnt < subsize; cnt += 1)
		forsort[origidx[cnt]] = origidx[cnt]
	endfor
	MakeIndex forsort, sortidx 
	for (cnt = 0; cnt < itemsinlist(parwaves); cnt += 1)
		wave thiswave = $stringfromlist(cnt,parwaves)
		wave /t thiswavetext = $stringfromlist(cnt,parwaves)
		string newwavename = sub_fullpath + ":" + stringfromlist(cnt,parwaves)
		
		if (WaveType(thiswave))	// if thiswave is numeric
			make /o /n = (subsize) /Y=(WaveType(thiswave)) $newwavename
			wave destwave = $newwavename
			destwave = thiswave[origidx[p]]
		else						// no it's a text wave
			duplicate /o thiswavetext, $newwavename
			wave /t destwavetext = $newwavename
			IndexSort sortidx, destwavetext
			DeletePoints subsize, (parsize - subsize), destwavetext
		endif
	endfor
	killwaves forsort, sortidx
	
// end alt method
#endif
	killwaves origidx // didn't use a free wave because I might decide to keep this around.  
	setdatafolder cdf	
End

// END PARAMETER SET MANIPULATIONS



// BEGIN GUI ELEMENTS

// Runs a dialog and a control panel to select molecules or isotopomers for inclusion in a parameters subset
Static Function HT_SelectSpeciesGUI()
	make /o /i /n = 0 root:Packages:HITRAN:temp:tempincludewave
	wave tempincludewave = root:Packages:HITRAN:temp:tempincludewave
	NVAR recentfreqlow = root:Packages:HITRAN:recentfreqlow
	NVAR recentfreqhigh = root:Packages:HITRAN:recentfreqhigh
//	make /FREE /i /n = 0 tempincludewave	// not working with a free wave because of execution of button procedure?
//	wave /z inc
	
	variable bymol_or_tope = 1
	prompt bymol_or_tope, "Select By", popup, "Molecule;Isotope"
	variable lowfreq = recentfreqlow, highfreq = recentfreqhigh
	prompt lowfreq, "Minimum wavenumber (cm-1) (0 for all)"
	prompt highfreq, "Maximum wavenumber (cm-1) (Inf for all)"
//	string originparfolder = "root:Packages:HITRAN:parfile"
	string originparfolder
	prompt originparfolder, "Original Parfile", popup, ParFolderList()
	DFREF cdf = GetDataFolderDFR()
	setdatafolder $HT_PARDF
	string destinationparfolder = UniqueName("subpar", 11, 0)
	setdatafolder cdf
	prompt destinationparfolder, "Create New Par Folder"
	variable intensitymin = 0
	prompt intensitymin, "Minimum Intensity"
	
	DoPrompt "Selection parameters", bymol_or_tope, lowfreq, highfreq, intensitymin, originparfolder, destinationparfolder
//	originparfolder = HT_PARDF + originparfolder
//	destinationparfolder = HT_PARDF + destinationparfolder
	if (V_flag)
		return 0
	endif
	destinationparfolder = QuoteLiberalName(destinationparfolder)
	if (datafolderexists(HT_PARDF + destinationparfolder))
		DoAlert 2, "Destination Folder Exists, Overwrite?"
		If (V_flag > 1)
			return 0
		endif
	endif
	bymol_or_tope -= 1
	variable okayed = SelectSpecies(tempincludewave,bymol_or_tope)		// Holds here via "PauseForUser" until user closes the control panel
	if (!okayed)
		return 0
	endif
	if (bymol_or_tope == 0)
		wave matchwave = $(HT_PARDF + originparfolder + ":molnum")
	else
		wave matchwave = $(HT_PARDF + originparfolder + ":moltope")
	endif
	MakeSubParFolder(lowfreq, highfreq, intensitymin, tempincludewave, matchwave, originparfolder,destinationparfolder)
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	S_recentparfolder = destinationparfolder
	recentfreqhigh = highfreq
	recentfreqlow = lowfreq
	killwaves /z tempincludewave
	return 1
End

// Works for now: improvements to add: default select cancel when hit return
// window hook function for panel to remove temp waves
// or better, move temp waves to a project folder.
Static Function SelectSpecies(incwave,selectby)
	wave incwave
	variable selectby // 0 = molecule, 1 = molecule & isotope
	switch (selectby)
		case 0:
//			wave /T objnames = root:Packages:HITRAN:general_data:molnum_code
			wave /T objnames = root:Packages:HITRAN:molparamdata:molnum_code
			break
		case 1:
			wave /T objnames = root:Packages:HITRAN:parsumdata:columnnames
			break
	endswitch
	variable numitems = numpnts(objnames)
	string incwavepnter = GetWavesDataFolder(incwave,2)
	make /o /b /n = (numitems) temp_selwave
//	make /o /b /n = (ceil(numitems/3),3) temp_selwave
	temp_selwave = 0	// in case it already exists, wipe bits
	temp_selwave += 0x20	// cells are checkboxes
	variable cnt
	for (cnt = 0; cnt < numpnts(incwave); cnt += 1)
		temp_selwave[incwave[cnt]] += 0x10
	endfor
	duplicate /o objnames, temp_listwave
	redimension /n = (ceil(numitems/3),3) temp_listwave,temp_selwave
//	string cdf = GetDataFolder(1)
//	NewDataFolder /o /s root:Packages:HITRAN:Packages
//	NewDataFolder /o /s root:Packages:HITRAN:Packages:HiTranSelector
//	variable /g dialogexitcode = 0
	NewPanel /N = Panel_SpeciesSelector /W=(263,92,633,559)
	ListBox ListBox_Molecules listwave = temp_listwave, selwave = temp_selwave, mode=4
	ListBox ListBox_Molecules,pos={1,1},size={350,400}
	Button Button_SelNone title="None", proc=ButtonProc_SpeciesSelector
	Button Button_SelAll title="All", proc=ButtonProc_SpeciesSelector
	Button Button_Okay title="Okay", proc=ButtonProc_SpeciesSelector, userdata = incwavepnter
	Button Button_Cancel title="Cancel", proc=ButtonProc_SpeciesSelector, userdata = incwavepnter
	PauseForUser Panel_SpeciesSelector
	return numpnts(incwave)	// length will be 0 if the panel was cancelled.
End

//     Finds all occurances of a value in datawave and stores results in foundatwave
//     Somewhat memory innefficient (initiallly redims foundatwave to be as large as datawave, then chops unused points at the end)

Static Function FindInWave(datawave, foundatwave, value)
	wave datawave, foundatwave
	variable value
	variable dataidx
	variable foundsofar = 0	
	redimension /n = (numpnts(datawave)) foundatwave
	for (dataidx = 0; dataidx < numpnts(datawave); dataidx += 1)
		if (datawave[dataidx] == value)
			foundatwave[foundsofar] = dataidx
			foundsofar += 1
		endif
	endfor
	redimension /n = (foundsofar) foundatwave
End

Function ButtonProc_SpeciesSelector(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			wave temp_selwave = temp_selwave
			strswitch (ba.ctrlName)
				case "Button_SelNone":
					temp_selwave = temp_selwave & ~0x10
					break
				case "Button_SelAll": 
					temp_selwave = temp_selwave | 0x10
					break
				case "Button_Okay":
					wave incwave = $ba.userdata
					temp_selwave = temp_selwave & 0x10	// wipe all info but checked or not
					FindInWave(temp_selwave,incwave,0x10)
					wave /t temp_listwave
					killwaves temp_selwave, temp_listwave
					KillWindow Panel_SpeciesSelector
					break
				case "Button_Cancel":
					wave incwave = $ba.userdata
					redimension /n = 0 incwave		// not really happy about this method of communication
					wave /t temp_listwave
					killwaves temp_selwave, temp_listwave
					KillWindow Panel_SpeciesSelector
					// for now no mode to userdata
					break
			endswitch
			break
	endswitch

	return 0
End


Static Function SA_MainPanel()
//	set up a panel to allow input of atmospheric consitutuents in terms of percent composition or
//	partial pressures.
	If (WinType("Panel_AtmosphericComposition"))
		DoWindow /F Panel_AtmosphericComposition
	else
		NewPanel /k=1 /W=(263,92,800,559) /N=Panel_AtmosphericComposition
	endif
	wave /t molnames = root:Packages:HITRAN:molparamdata:molnum_code
	wave molfracs = root:Packages:HITRAN:externaldata:molfraction
	variable cnt
	SetVariable SV_0 title="Others", value = molfracs[0], size = {100,20}, limits={0,1,0.01 }
//	wave Ctab = root:Packages:HITRAN:HT_ctabmol
	for (cnt = 1; cnt < numpnts(molnames) ; cnt += 1)
		SetVariable $("SV_" + num2str(cnt)) title = molnames[cnt], value = molfracs[cnt], size = {100,20}, limits={0,1,0.01 }
//		SetVariable $("SV_" + num2str(cnt)) valueBackColor = (Ctab[cnt][0],Ctab[cnt][1],Ctab[cnt][2])
	endfor
	Button Button_Norm title="ReNormalize", size = {150,20}, proc=HITRAN#SA_ButtonHandler
	Button Button_SetToZero title = "Set All to Zero", size = {150,20}, proc=HITRAN#SA_ButtonHandler
	Button Button_SetToStandard title = "Set To Standard", size =  {150,20}, proc=HITRAN#SA_ButtonHandler
End

Static Function SA_ButtonHandler(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	switch( ba.eventCode )
		case 2: // mouse up
			wave molfracs = root:Packages:HITRAN:externaldata:molfraction
			wave molfracs_standard = root:Packages:HITRAN:externaldata:molfraction_standard
			strswitch (ba.ctrlName)
				case "Button_Norm":
					variable thesum = sum(molfracs)
					molfracs /= thesum
					break
				case "Button_SetToZero":
					molfracs = 0
					break
				case "Button_SetToStandard":
					molfracs = molfracs_standard
			EndSwitch
	EndSwitch	
End

// Generates the list of datafolders within the PARSets package folder.  Reads a global variable to place
//  the most recently used par set at the top of the list (and thus the default value for any popup prompt using this list)
//  To handle liberal names (all the by molecule *.par files start with a number) it was easiest to just have this 
// function surround any liberal names with single quotes in building the list, which will show up in a popup menu but also makes for 
/// simpler selection parsing.  Also occasionally misses the unquoted name after a new liberally named DF is created, but only the first time. 

Static Function /s ParFolderList()
	string cdf = GetDataFolder(1)
	if (!DataFOlderExists(HT_PARDF))
		return ""
	endif
	SVAR /z recentparfolder = root:Packages:HITRAN:S_recentparfolder
	string thelist = ""
	string thisfolder = ""
	variable cnt = 0
	setdatafolder $HT_PARDF
	do
		thisfolder = GetIndexedObjName(":",4,cnt)
		if (!strlen(thisfolder))
			break
		endif
		if (!stringmatch(thisfolder,CleanUpName(thisfolder,0)))  	// liberal name, need to add single quotes
			thisfolder = "'" + thisfolder + "'"
		endif
		thelist = AddListItem(thisfolder, thelist, ";", Inf)
		cnt += 1
	while(1)
	if (SVAR_exists(recentparfolder))
		if (DataFolderExists("root:Packages:HITRAN:PARSets:" + recentparfolder))
			variable recentidx = whichlistitem(recentparfolder,thelist)
			thelist = RemoveListItem(recentidx,thelist)
			thelist = AddListItem(recentparfolder,thelist,";",0)
		endif
	endif
	setdatafolder cdf
	return thelist
End

// Why did I have to write these?  There's got to be a better way to quote or unquote intelligently.  
//
// In both cases they only quote or unquote as needed to make a liberal name work.  
Static Function /S UnQuoteLiberalName(objectname)
	string objectname
	if (stringmatch(objectname,"'*'"))
		return objectname[1,strlen(objectname) - 2]
	else
		return objectname
	endif
End

Static Function /S QuoteLiberalName(objectname)
	string objectname
	if (stringmatch(objectname,"'*'"))
		return objectname						// was already quoted
	endif
	if (stringmatch(CleanUpName(objectname,0),objectname))
		return objectname						// doesn't need single quotes
	endif
		
	return "'" + objectname + "'"					// single quotes added
End


// New ColorCodePanel allows user to select desired color coding by molecule

Static Function ColorCodePanel()
	If (WinType("Panel_ColorCodes"))
		DoWindow /F Panel_ColorCodes
//		return 1		// Check that...re-run control commands will update colors if HT_Ctabmol has been modified by any other method.  
	else
		NewPanel /k=1 /W = (20,20,410,600) /N=Panel_ColorCodes
	endif
//	string cdf = GetDataFolder(1)
	wave /t MoleculeNames = root:Packages:HITRAN:molparamdata:molnum_code
	wave HT_ctabmol = root:Packages:HITRAN:HT_ctabmol
	variable cnt
	string ctrlname, titlestr
	variable vpos, hpos
	variable numrows = ceil(numpnts(MoleculeNames) / 2 )
	for (cnt = 0; cnt < numpnts(MoleculeNames); cnt +=1 )
//		vpos = trunc(cnt / 2) * 20 + 10
//		hpos = mod(cnt, 2) *150 + 10
		vpos = mod(cnt, numrows) * 20 + 10
		hpos = trunc(cnt / numrows) * 180 + 10
		ctrlname = "pop_molcode_" + num2str(cnt)
		sprintf titlestr, "%02d  ", cnt
		titlestr = titlestr + PadString(MoleculeNames[cnt], 7, 0x20)
		PopupMenu $ctrlname font = "Courier" // use with padstring for text alignment, meed a monotype
		PopupMenu $ctrlname title=titlestr, popColor = (HT_ctabmol[cnt][0],HT_ctabmol[cnt][1],HT_ctabmol[cnt][2]), value = "*COLORPOP*"
		PopupMenu $ctrlname size={110,15},pos={hpos,vpos}, proc = HITRAN#ColorCode_PopMenuProc
	endfor
End

Static Function ColorCode_PopMenuProc(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	switch( pa.eventCode )
		case 2: // mouse up
			Variable popNum = pa.popNum
			String popStr = pa.popStr
			wave HT_ctabmol = root:Packages:HITRAN:HT_ctabmol
			variable V_Red, V_Green, V_Blue, V_molcode
			sscanf popStr, "(%d,%d,%d)", V_Red, V_Green, V_Blue
			sscanf pa.CtrlName, "pop_molcode_%d", V_molcode
			HT_ctabmol[V_molcode][0] = V_Red
			HT_ctabmol[V_molcode][1] = V_Green
			HT_ctabmol[V_molcode][2] = V_Blue
			break
		case -1: // control being killed
			break
	endswitch

	return 0
End


// END GUI ELEMENTS





// BEGIN CALCULATIONS


// Creates waves for line information that are dependent on pressure and temperature:
// VarFreq is always created, and holds pressure shifted frequencies.
// VarInten and/or VarEmit are created and hold pressure and temperature dependent absorption intensity and/or emission rate data.  VarInten is 
// equal to Inten at 296K, 1 atm, and with a molfraction of 1 for the molecule of interest.  VarEmit is actually based on EinstA data but in different units
// (photons / s / molecule / wn) which is dependent on sample content.
//
//  Additionally, linewidth info is calculated, and depending on broadening phenomena modeled will include
//  LorWidth (pressure broadening)
//  GaussWidth (doppler broadening)
//
// molfractions and relpartition are optional wave references that will point to default package globals if not passed explicitly.  Right now there is no way to setup multiple atmospheres with the GUI but 
// if you wanted to program a layered calculation with a layer dependent atmosoheric composition, individual layer data would be passed to this function via the optional molfractions argument
//
// The optional partmat is for users calculating their own partition function data.  The HITRAN website supplies fortran code for calculating partition functions, which I personally have not experimented with
// 
//
// Some shared overhead from calculating emission and absorption at the same time (frequency and linewidth, to be precise), so 
// whichinten = 1 for absorption (called VarInten), 2 for emission, 3 for both.  If both emission and absorption data are ultimately being calculated, 
// it is faster to calculate VarInten and VarEmit in a single function call (this will be especially noticeable for a very large par set with a small wavelength range calculation)
//
// whichwidths = 1 for Pressure, 2 for Doppler, 3 for both
//  Not implemented yet, but might include  4 for Radiative Lifetime (Also contributes to Lorentzian component, so just needs to be calced)?
//  Typically radiative lifetime in the IR will be a non-issue compared to pressure and doppler broadening, and calculating is difficult with given data
//  because EinstA coefficients of ALL transitions from a common upper state set the radiative lifetime, not just the line in question.  
//  Calculated widths are half-widths (at half max).
//
// Need to fix temperature interpolation.  Right now up to 2% error for non-integer temperatures because wave indexing doesn't interpolate with 2D matrices the
// way it does with 1D waves (my mistake...I knew that, but forget when implementing the partition function lookup).  
//  Partition Data is supplied in 1 K increments.  The largest error (2%) occurs at low temperatures and values just below an integer (e.g. partition function at
// 70.99 K gets truncated to 70K value)
//
// Added return value to indicate lack of atmospheric content.  Return 1 is normal operation, Return 2 is non-error, but indicates VarInten full of 0's, almost certainly because the parameter
// set is a single molecule which is not normally in the atmosphere and the user has not set the atmosphere yet.  Checks by summing either VarInten or VarEmit (if neither is requested it will perhaps misleadingly
// indicate a lack of population, but since the linewidths don't need molfractions to calculate, no real harm)
Function HT_CalcVarLineData(parfolder, pressure, temp, whichinten, whichwidths, [molfractions, partmat])
	string parfolder
	variable pressure, temp, whichinten, whichwidths
	wave /z molfractions
	wave /z partmat
	DFREF cdf = GetDataFolderDFR()
	setdatafolder $(HT_PARDF + parfolder)
	wave freq, intensity, gamma_air, gamma_self, EngGS, molnum, delta_air, n_air, EinstA, gprime, moltope
	variable numlines = numpnts(freq)
	variable nopopulation = 1
	if (!waveexists(molfractions))	
		wave molfractions = root:Packages:HITRAN:externaldata:molfraction
	endif
	if (!waveexists(partmat))
		wave partmat = root:Packages:HITRAN:parsumdata:partition_mat
	endif	

	make /d /free /n = (DimSize(partmat,1)) relpartition_row, part_row
	relpartition_row = partmat(temp)[p] / partmat(296)[p]					// Find partition function (for emit) and relative partition function (for absorbance intensity) at temp.
	part_row = partmat(temp)[p]										// Use of 2D matrix for partition function prohibits linear interpolation at temp.  
	
	wave isoabund = root:Packages:HITRAN:molparamdata:Abundance			// abundance of each isotopomer as fraction of that chemical species
	make /o /d /n = (numlines) VarFreq

	Multithread VarFreq = freq + delta_air * pressure
	if (whichwidths & 1)
		make /o /d /n = (numlines) LorWidth
		Multithread LorWidth = pressure*((1 - molfractions[molnum])*gamma_air + molfractions[molnum]*gamma_self)*(296/temp)^n_air
	endif
	if (whichwidths & 2)
		make /o /d /n = (numlines) GaussWidth
		wave molarmass = root:Packages:HITRAN:molparamdata:molarmass 		
		Multithread GaussWidth = VarFreq*sqrt(KAYINJOULES*temp / (molarmass[moltope[p]]*AMU2KG*CINMS^2))*GSIGTOFWHM/2 		
	endif
	if (whichinten & 1)
		make /o /d /n = (numlines) VarInten
		Multithread VarInten = intensity/relpartition_row[moltope]*exp(-EngGS/KAYINWN * (1/temp - 1/296))*((1-exp(-VarFreq/KAYINWN/temp))/(1-exp(-VarFreq/KAYINWN/296)))*molfractions[molnum]
		if (sum(VarInten) > 0)
			nopopulation = 0
		endif
	endif
	// see HITRAN 96 documentation for this explanation (eq. A11).  Basically intensities are reported for atmospheric pressure and 296K.
	// intensity adjustment for temperature requires a few ratios: partition function accounts ts for state dilution, exp(-EngGS...) accounts for 
	// Boltzman population effects in the starting state, and ext(-freq...) accounts for stimulated emission / reradiance of low energy 
	//
	// Note that isotopic abundances are factored into the intensity column loaded from the HITRAN database, so are not explicitly included in VarInten calculation 
	if (whichinten & 2)
		make /o /d /n = (numlines) VarEmit
		Multithread VarEmit = gprime*EinstA*molfractions[molnum]*isoabund[moltope]*exp(-(EngGS + freq)/(temp * KAYINWN)) / part_row[moltope[p]]
		if (sum(VarEmit) > 0)
			nopopulation = 0
		endif

	endif
	// Note isoabund is needed for Emission Calc because it's not built up from the original intensity wave


//    // Using MatrixOp: was an improvement over simple assignment, but not so much with MultiThread assignments.  Have to pre-determine the lookup wave values (*_byline)
//    // which kills a lot of the speed gain from MatrixOP unless I've missed a way to use a wave for a point index in MatrixOp

//	make /free /d /n = (numpnts(freq)) molfrac_byline, partition_byline, relpart_byline, isoabund_byline
//	matrixop /o VarFreq = freq + delta_air * pressure
	
//	molfrac_byline = molfractions[molnum[p]]
//	partition_byline = partmat(temp)[moltope[p]]
//	relpart_byline = relpartition(temp)[moltope[p]]
//	isoabund_byline = isoabund[moltope[p]]

//	matrixop /o LorWidth = pressure*((1 + (-1*molfrac_byline))*gamma_air + molfrac_byline * gamma_self)*powR(296 / temp,n_air)	
	
//	variable tempfac = 1/temp - 1/296	
//	matrixop /o VarInten = intensity / relpart_byline * exp(-EngGS / 0.69503476 * tempfac) *  (exp(-1*freq/(0.69503476*temp)) - 1) / (exp(-1*freq/(0.69503476*296)) - 1) * molfrac_byline


//	matrixop /o VarEmit = gprime * EinstA * molfrac_byline * isoabund_byline * exp(-1*(EngGS + freq) / (temp * 0.69503476)) / partition_byline

	setdatafolder cdf
	if (nopopulation)
		return 2		// flag to warn that Variable Intensity / Emission wave(s) are full of 0s, probably because user did not update the atmosphere to include population of molecule in the parameter set.
	else
		return 1
	endif
End

Proc HT_CalcVarLineDataWrapper(pressure, temp, parfolder)
	variable pressure = root:Packages:HITRAN:recentpress
	prompt pressure, "Pressure (atm)"
	variable temp = root:Packages:HITRAN:recenttemp
	prompt temp, "Temperature (K)"
	string parfolder
	prompt parfolder, "Parameters Folder", popup, HITRAN#ParFolderList()
//	ModifyLorCoefs(parfolder,pressure,temp,3)
	HITRAN#HT_CalcVarLineData(parfolder, pressure, temp, 3, 3)
	root:Packages:HITRAN:S_recentparfolder = parfolder
	root:Packages:HITRAN:recentpress = pressure
	root:Packages:HITRAN:recenttemp = temp
End


// estimates a weighted line width and uses that for the spectral broadening.  This is the "fast" calculation because it can take adantage of a fast convolution algorith.  
// For now only Voigt profile allowed (not really a speed problem, but perhaps for consistancy should include option for gauss or lor only)

Static function HT_CalcBandsOneWidth(freqlow, freqhigh, wnspacing, pressure, temp, parfolder, abswavenm,emwavenm,transwavenm)
	variable freqlow, freqhigh, wnspacing, pressure, temp
	string parfolder, abswavenm, emwavenm, transwavenm
//	ModifyVoightCoefs(parfolder,pressure,temp,3)
	HT_CalcVarLineData(parfolder, pressure, temp, 3, 3)	// parse emission / absorption ala accurate calc.
	string cdf = GetDataFOlder(1)
	variable numpoints = (freqhigh - freqlow) / wnspacing + 1
	setdatafolder $(HT_PARDF + parfolder)
	wave VarFreq, VarInten, LorWidth, VarEmit, GaussWidth
	setdatafolder cdf
	variable Lwidth = CalcAverageLineWidth(parfolder, 1, freqlow, freqhigh)
	variable Gwidth = CalcAverageLineWidth(parfolder, 0, freqlow, freqhigh)
	if (temp < 70) 
		print "Caution: Partition matrix only valid to 70K.  Spectral shape should be okay, but magnitude is off."
	endif

	if (strlen(abswavenm))
		make /o /d /n = (numpoints) $abswavenm
		wave abswave = $abswavenm
		setscale /p x, (freqlow), (wnspacing), abswave
		abswave = 0
		FillDestWave(VarFreq, VarInten, abswave, fillmethod = DEFAULT_FILL_METHOD)
		VoigtConvolve(abswave,Lwidth,Gwidth)
		note /K abswave, "Absorptivity Units are cm^2 / molecules"
		note abswave, ("Pressure = " + num2str(pressure) + " atm")
		note abswave, ("Temperature = " + num2str(temp) + " K")

		if (strlen(transwavenm))
			duplicate /o abswave $transwavenm
			wave transwave = $transwavenm
			transwave = exp(-abswave*pressure/temp*PRESSTODENS)
			note /K transwave, "Transmission is unitless, represents 1cm travel"
			note transwave, ("Pressure = " + num2str(pressure) + " atm")
			note transwave, ("Temperature = " + num2str(temp) + " K")
		endif
	endif
	if (strlen(emwavenm))
		make /o /d /n = (numpoints) $emwavenm
		wave emwave = $emwavenm
		setscale /p x, (freqlow), (wnspacing), emwave
		emwave = 0
		FillDestWave(VarFreq, VarEmit, emwave, fillmethod = DEFAULT_FILL_METHOD)
		VoigtConvolve(emwave,Lwidth,Gwidth)
		note /K emwave, "Emission units are Photons / molecule / cm^-1 / second"		
		note emwave, ("Pressure = " + num2str(pressure) + " atm")
		note emwave, ("Temperature = " + num2str(temp) + " K")

	endif
End

Proc HT_CalcBandsOneWidthWrapper(freqlow, freqhigh, wnspacing, pressure, temp, parfolder, abswavenm,emwavenm,transwavenm)
	variable freqlow = root:Packages:HITRAN:recentfreqlow
	prompt freqlow, "Low Frequency"
	variable freqhigh = root:Packages:HITRAN:recentfreqhigh
	prompt freqhigh, "High Frequency"
	variable wnspacing = root:Packages:HITRAN:recentwnspacing
	variable pressure = root:Packages:HITRAN:recentpress
	prompt pressure, "Pressure (Atm)"
	variable temp = root:Packages:HITRAN:recenttemp
	prompt temp, "Temperature (K)"
	string parfolder = "root:Packages:HITRAN:parfile"
	prompt parfolder, "Parameters Folder", popup, HITRAN#ParFolderList()
	string abswavenm = "absorb"
	prompt abswavenm, "Name of Wave for Absorbance Output"
	string emwavenm = ""
	prompt emwavenm, "Name of Wave for Emission Output"
	string transwavenm = ""
	prompt transwavenm, "Name of Wave for Transmission Output"
	HITRAN#HT_CalcBandsOneWidth(freqlow, freqhigh, wnspacing, pressure, temp, parfolder, abswavenm,emwavenm,transwavenm)
	root:Packages:HITRAN:recentpress = pressure
	root:Packages:HITRAN:recenttemp = temp
	root:Packages:HITRAN:S_recentparfolder = parfolder
	root:Packages:HITRAN:recentfreqlow = freqlow
	root:Packages:HITRAN:recentfreqhigh = freqhigh
	root:Packages:HITRAN:recentwnspacing = wnspacing
End

#if Exists("LorentzianFit")		// Looking for installation of MultipeakFitting XOP which defines LorentzianFit (and GaussianFit)

// optional molfractions and partmat point to default package globals if unspecified (controlled by the CalcLines Function)

Static Function HT_CalcBandsAccurate(freqlow, freqhigh, deltafreq, pressure, temp, whichwidths, parfolder, abswavenm,emwavenm,transwavenm,[molfractions, partmat])
	variable freqlow, freqhigh, deltafreq, pressure, temp, whichwidths
	string parfolder, abswavenm, emwavenm, transwavenm
	wave /z molfractions, partmat
	DFREF cdf = GetDataFolderDFR()
	variable numpoints = (freqhigh - freqlow) / deltafreq + 1
	variable whichinten = 0
	if (strlen(abswavenm))
		make /o /d /n = (numpoints) $abswavenm
		wave abswave = $abswavenm
		setscale /P x, (freqlow),(deltafreq),"", abswave
		whichinten += 1
		note /K abswave, "Absorptivity Units are cm^2 / molecules"
		note abswave, ("Pressure = " + num2str(pressure) + " atm")
		note abswave, ("Temperature = " + num2str(temp) + " K")
	endif
	if (strlen(emwavenm))
		make /o /d /n = (numpoints) $emwavenm
		wave emitwave = $emwavenm
		setscale /P x, (freqlow),(deltafreq),"", emitwave
		whichinten += 2
		note /K emitwave, "Emission units are Photons / molecule / cm^-1 / second"		
		note emitwave, ("Pressure = " + num2str(pressure) + " atm")
		note emitwave, ("Temperature = " + num2str(temp) + " K")

	endif
	HT_CalcVarLineData(parfolder, pressure, temp, whichinten, whichwidths, molfractions = molfractions, partmat = partmat)
	setdatafolder $(HT_PARDF + parfolder)
	wave VarFreq
	variable numlines = numpnts(VarFreq)
	switch (whichwidths)
		case 0:
			print "Need some kind of line broadening"
			return 0
		case 1:				// Pressure only  - Lorentzian
			wave LorWidth
			make /d /free /n = (numlines) LorWidth_sq
			Multithread LorWidth_sq = LorWidth^2			
			if (strlen(abswavenm))
				wave VarInten
				make /d /free /n = (numlines) VarInten_LorNorm
				Multithread VarInten_LorNorm = VarInten * LorWidth / pi
				concatenate /o {VarInten_LorNorm,VarFreq,LorWidth_sq}, LorAbsCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose LorAbsCoefs
				redimension /n = (numpnts(LorAbsCoefs)) LorAbsCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, LorAbsCoefs						// insert first point for baseline.  
				abswave = LorentzianFit(LorAbsCoefs,x)				// This would benefit from Multithreading, but Multipeak Fit Functions are not marked Threadsafe
				killwaves /z LorAbsCoefs
			endif
			if (strlen(emwavenm))
				wave VarEmit
				make /d /free /n = (numlines) VarEmit_LorNorm
				Multithread VarEmit_LorNorm = VarEmit * LorWidth / pi
				concatenate /o {VarEmit_LorNorm,VarFreq,LorWidth_sq}, LorEmitCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose LorEmitCoefs
				redimension /n = (numpnts(LorEmitCoefs)) LorEmitCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, LorEmitCoefs						// insert first point for baseline.  
				emitwave = LorentzianFit(LorEmitCoefs,x)
				killwaves /z LorEmitCoefs
			endif	
			break
		case 2:				// Doppler only   - Gaussian (Typically I would do Doppler only with the fast approximation.  But whatever..)
			wave GaussWidth
			make /d /free /n = (numlines) GaussSigma
			Multithread GaussSigma = GaussWidth * (sqrt(2) * 2 / GSIGTOFWHM)	// poor name...it's actually sqrt(2) * sigma, but it's what goes into GaussFit 			
			if (strlen(abswavenm))
				wave VarInten
				make /d /free /n = (numlines) VarInten_GaussNorm
				Multithread VarInten_GaussNorm = VarInten / GaussSigma / sqrt(pi)		// check this
				concatenate /o {VarInten_GaussNorm,VarFreq,GaussSigma}, GaussAbsCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose GaussAbsCoefs
				redimension /n = (numpnts(GaussAbsCoefs)) GaussAbsCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, GaussAbsCoefs						      // insert first point for baseline.  
				abswave = GaussFit(GaussAbsCoefs,x)				// This would benefit from Multithreading, but Multipeak Fit Functions are not marked Threadsafe
				killwaves /z GaussAbsCoefs
			endif
			if (strlen(emwavenm))
				wave VarEmit
				make /d /free /n = (numlines) VarEmit_GaussNorm
				Multithread VarEmit_GaussNorm = VarEmit / GaussSigma / sqrt(pi)	
				concatenate /o {VarEmit_GaussNorm,VarFreq,GaussSigma},GaussEmitCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose GaussEmitCoefs
				redimension /n = (numpnts(GaussEmitCoefs)) GaussEmitCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, GaussEmitCoefs						// insert first point for baseline.  
				emitwave = GaussFit(GaussEmitCoefs,x)
				killwaves /z GaussEmitCoefs
			endif	
			break

		case 3:				// Pressure and Doppler  - Voigt
			wave LorWidth, GaussWidth
			make /d /free /n = (numlines) Voigt2, Voigt4
			Multithread Voigt2 = sqrt(ln(2)) / GaussWidth
			Multithread Voigt4 = LorWidth * Voigt2
//			Multithread Voigt3 = VarFreq
// See help for VoightFit to see derivation of conversion from physical parameters to VoightFit parameters
			if (strlen(abswavenm))
				wave VarInten
				make /d /free /n = (numlines) Voigt1_abs
				Multithread Voigt1_abs = VarInten * Voigt2 / sqrt(pi)
				concatenate /o {Voigt1_abs,Voigt2,VarFreq,Voigt4}, VoigtAbsCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose VoigtAbsCoefs
				redimension /n = (numpnts(VoigtAbsCoefs)) VoigtAbsCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, VoigtAbsCoefs						// insert first point for baseline.  
				abswave = VoigtFit(VoigtAbsCoefs,x)				// This would benefit from Multithreading, but Multipeak Fit Functions are not marked Threadsafe
				killwaves /z VoigtAbsCoefs
			endif
			if (strlen(emwavenm))
				wave VarEmit
				make /d /free /n = (numlines) Voigt1_emit
				Multithread Voigt1_emit = VarEmit * Voigt2 / sqrt(pi)
				concatenate /o {Voigt1_emit,Voigt2,VarFreq,Voigt4}, VoigtEmitCoefs	// no /Free flag on concatenate...could I do this better with matrixop?
				matrixtranspose VoigtEmitCoefs
				redimension /n = (numpnts(VoigtEmitCoefs)) VoigtEmitCoefs	// seems odd, but remember numpnts is stretched length so this works to convert to 1D wave.
				insertpoints 0, 1, VoigtEmitCoefs						// insert first point for baseline.  
				emitwave = VoigtFit(VoigtEmitCoefs,x)				// This would benefit from Multithreading, but Multipeak Fit Functions are not marked Threadsafe
				killwaves /z VoigtEmitCoefs
			endif	
	
		
	EndSwitch
	setdatafolder cdf
	if (strlen(abswavenm) && strlen(transwavenm))
		make /d /o /n = (numpoints) $transwavenm
		wave transwave = $transwavenm		
		CopyScales abswave, transwave
		transwave = exp(-abswave*pressure/temp*PRESSTODENS)
		note /K transwave, "Transmission is unitless, represents 1cm travel"
		note transwave, ("Pressure = " + num2str(pressure) + " atm")
		note transwave, ("Temperature = " + num2str(temp) + " K")
	endif
	
	return 1

End

// Wrapper proc for HT_CalcBandsAccurate(...)
Proc HT_CalcBandsWrapper(freqlow, freqhigh, freqspacing, pressure, temp, parfolder, abswavenm,emwavenm,transwavenm,broadening)
	variable freqlow = root:Packages:HITRAN:recentfreqlow
	variable freqhigh = root:Packages:HITRAN:recentfreqhigh
	variable freqspacing = root:Packages:HITRAN:recentwnspacing
	prompt freqspacing, "Output point sampling (cm-1)"
	variable pressure = root:Packages:HITRAN:recentpress
	prompt pressure, "Pressure (atm)"
	variable temp = root:Packages:HITRAN:recenttemp
	prompt temp, "Temperature (K)"
	string parfolder
	prompt parfolder, "Parameters Folder", popup, HITRAN#ParFolderList()
	string abswavenm = "absorb"
	prompt abswavenm, "Name of Wave for Absorbance Output"
	string emwavenm = ""
	prompt emwavenm, "Name of Wave for Emission Output"
	string transwavenm = ""
	prompt transwavenm, "Name of Wave for Transmission Output"
	variable broadening = 1
	prompt broadening, "Broadening", popup, "Pressure Only;Doppler Only; Pressure and Doppler"
//	if (temp < 70) 
//		DoAlert, 0, "Caution: Partition matrix only valid to 70K.  Spectral shape should be okay, but magnitude is off"
//	endif
//	variable whichwidths = (broadening == 1) ? 1 : 3 		// for now...perhaps more options in the future with a proper switch statement
	HITRAN#HT_CalcBandsAccurate(freqlow, freqhigh, freqspacing, pressure, temp, broadening, parfolder, abswavenm,emwavenm,transwavenm)
	root:Packages:HITRAN:S_recentparfolder = parfolder
	root:Packages:HITRAN:recentpress = pressure
	root:Packages:HITRAN:recenttemp = temp
	root:Packages:HITRAN:recentfreqlow = freqlow
	root:Packages:HITRAN:recentfreqhigh = freqhigh
	root:Packages:HITRAN:recentwnspacing = freqspacing
End

#else
Proc HT_CalcBandsWrapper()
	DoAlert 0, "You Must Install MultPeakFitting XOP to run accurate calculations"
End
#endif


static Function CA_CalculationAdvisorPanel()
	DoWindow /F PanelAdvisor
	if (V_Flag)
		return 1
	endif
	DFREF cdf = GetDataFolderDFR()
	setdatafolder root:Packages:HITRAN
	NVAR recenttemp, recentpress, recentfreqlow, recentfreqhigh, recentwnspacing
	SVAR S_recentparfolder

	NewPanel /K=1 /N=PanelAdvisor /W=(730,75,1389,629) as "Calculation Advisor"
	SetDrawLayer UserBack
	DrawText 27,86,"Simulation Range"
	DrawText 27,250,"Pressure Broadening (cm\\S-1\\M)"
	DrawText 27,331,"Doppler Broadening (cm\\S-1\\M)"
	DrawText 402,249,"Line Width Histogram"
	SetVariable setvar_temp,pos={27,16},size={166,16},proc=HITRAN#CA_SetVar_Simsettings,title="Temperature (K)"
	SetVariable setvar_temp,value= root:Packages:HITRAN:recenttemp
	SetVariable setvar_pressure,pos={27,43},size={163,16},proc=HITRAN#CA_SetVar_Simsettings,title="Pressure (atm)"
	SetVariable setvar_pressure,value= root:Packages:HITRAN:recentpress
	SetVariable setvar_lowf,pos={27,93},size={211,24},proc=HITRAN#CA_SetVar_Simsettings,title="Low Frequency (cm\\S-1\\M)"
	SetVariable setvar_lowf,value=  root:Packages:HITRAN:recentfreqlow
	SetVariable setvar_highf,pos={27,120},size={211,24},proc=HITRAN#CA_SetVar_Simsettings,title="High Frequency (cm\\S-1\\M)"
	SetVariable setvar_highf,value= root:Packages:HITRAN:recentfreqhigh
	PopupMenu popup_parset,pos={318,14},size={201,21},proc=HITRAN#CA_PopMenuProc_ParSet,title="Parameters Set"
	PopupMenu popup_parset,mode=1,value= #"HITRAN#ParFolderList()"
	Button button_Evaluate,pos={27,497},size={134,27},proc=HITRAN#CA_ButtonProc_Update,title="Update Line Parameters"
	ValDisplay valdisp_numLinesrange,pos={27,152},size={181,14},title="Number of Lines"
	ValDisplay valdisp_numLinesrange,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_parlowf,pos={320,96},size={226,14},title="Lowest Frequency In set"
	ValDisplay valdisp_parlowf,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_parhighf,pos={320,126},size={224,14},title="High Frequency"
	ValDisplay valdisp_parhighf,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_numLinespar,pos={320,153},size={124,14},title="# Lines"
	ValDisplay valdisp_numLinespar,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_numNoGSrange,pos={27,179},size={198,14},title="Missing Ground States"
	ValDisplay valdisp_numNoGSrange,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_numNoGSpar,pos={318,176},size={183,14},title="# Missing Ground State"
	ValDisplay valdisp_numNoGSpar,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_numNoArange,pos={27,206},size={185,14},title="Missing Einstein A"
	ValDisplay valdisp_numNoArange,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_numNoApar,pos={321,200},size={184,14},title="# Missing Einstein A"
	ValDisplay valdisp_numNoApar,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	CheckBox check_LU,pos={27,476},size={76,14},title="Live Update",value= 0
	ValDisplay valdisp_RecWnSpace,pos={27,392},size={232,20},title="Recommended Spacing (cm\\S-1\\M)"
	ValDisplay valdisp_RecWnSpace,limits={0,0,0},barmisc={0,1000},value= _NUM:0,  format="%4.3g"
	ValDisplay valdisp_numcalcsaccurate,pos={27,449},size={215,14},title="# Calculations"
	ValDisplay valdisp_numcalcsaccurate,limits={0,0,0},barmisc={0,1000}
	ValDisplay valdisp_numcalcsaccurate,value= _NUM:0
	ValDisplay valdisp_outputsize,pos={27,422},size={222,14},title="Output Wave Points"
	ValDisplay valdisp_outputsize,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_avgLwidth,pos={49,257},size={220,14},title="Weighted Mean",  format="%4.3g"
	ValDisplay valdisp_avgLwidth,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_AvgDwidth,pos={52,338},size={217,14},title="Weighted Mean",  format="%4.3g"
	ValDisplay valdisp_AvgDwidth,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	Button button_SetAtmosphere,pos={319,49},size={111,26},proc=HITRAN#CA_ButtonProc_SetAtmopshere,title="Set Atmosphere"
	Button button_SetAtmosphere,fColor=(65535,65535,65535)
	ValDisplay valdisp_minLwidth,pos={50,287},size={116,14},title="Minimum",  format="%4.3g"
	ValDisplay valdisp_minLwidth,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	ValDisplay valdisp_minDwidth,pos={55,368},size={117,14},title="Minimum",  format="%4.3g"
	ValDisplay valdisp_minDwidth,limits={0,0,0},barmisc={0,1000},value= _NUM:0
	Button button_Lwidth_showhist,pos={183,285},size={87,20},proc=HITRAN#CA_ButtonProc_Hist,title="Histogram"
	Button button_Dwidth_showhist,pos={183,365},size={88,20},proc=HITRAN#CA_ButtonProc_Hist,title="Histogram"
	Display /N = histsub /W=(296,253,623,521)/HOST=PanelAdvisor  
	CA_SelParSet(StringFromList(0,ParFolderList()))
	setdatafolder cdf
	return 1
End

static Function CA_SetVar_Simsettings(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			ControlInfo /W=$sva.Win check_LU
			if (V_Value)					// If "Live Update" is checked...
				CA_PanelUpdate(PanelName = sva.Win)
			endif			
		break
		case -1: // control being killed
			break
	endswitch

	return 0
End

static Function CA_ButtonProc_Update(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	if (ba.eventCode == 2)			// mouse up
		CA_PanelUpdate(PanelName = ba.Win)
	endif
	return 0
End

static Function CA_ButtonProc_Hist(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	if (ba.eventCode == 2)			// mouse up
		DFREF cdf = GetDataFolderDFR()
		ControlInfo /W=$ba.win popup_parset
		string parfolder = S_Value
		setdatafolder $(HT_PARDF + parfolder)
		wave /z VarInten, VarFreq
		if (!waveexists(VarInten))
			setdatafolder cdf
			return 0
		endif
		ControlInfo  /W=$ba.win setvar_lowf
		variable lowidx = BinarySearch(VarFreq,V_Value)
		ControlInfo  /W=$ba.win setvar_highf
		variable highidx = BinarySearch(VarFreq,V_Value)
		highidx = (highidx == -2) ? numpnts(VarFreq) - 1 : highidx		// outofrange high with last point
		variable numlines
		if ((lowidx == -2) || (highidx == -1) || (lowidx == -3))
			setdatafolder cdf
			return 0
		endif

		
		strswitch (ba.CtrlName)
			case "button_Lwidth_showhist":
				wave /z LorWidth
				if (!waveexists(LorWidth))
					setdatafolder cdf
					return 0
				endif
				make /o /d /n = 0 LorWidth_Hist
				Histogram /B=4 /R=[lowidx,highidx] /W=VarInten LorWidth, LorWidth_Hist
//				Display /k = 1 LorWidth_Hist
				wave dispwave = LorWidth_Hist
				break
			case "button_Dwidth_showhist":
				wave /z GaussWidth
				if (!waveexists(GaussWidth))
					setdatafolder cdf
					return 0
				endif
				make /o /d /n = 0 GaussWidth_Hist
				Histogram /B=4 /R=[lowidx,highidx] /W=VarInten GaussWidth, GaussWidth_Hist
//				Display /k = 1 GaussWidth_Hist
				wave dispwave = GaussWidth_Hist
				break
		endswitch
		string thelist = TraceNameList(ba.win + "#HistSub", ";",  1)
		variable cnt = 0
		for (cnt = 0; cnt < itemsinlist(thelist); cnt += 1)
			RemoveFromGraph /W=$(ba.win + "#HistSub") $stringfromlist(cnt,thelist)
		endfor
		AppendToGraph /W=$(ba.win + "#HistSub") dispwave
		Label /W=$(ba.win + "#HistSub" ) left "Weighted Frequency"
		Label /W=$(ba.win + "#HistSub")  bottom "Width (cm\\S-1\\M)"		 
		setdatafolder cdf
	endif
	return 0
End

static Function CA_PopMenuProc_ParSet(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	if (pa.eventCode == 2) 	// mouse up
		CA_SelParSet(pa.popStr, PanelName = pa.win)
//		String parfolder = pa.popStr
//		String PanelName = pa.win
//		DFREF cdf = GetDataFolderDFR()
//		setdatafolder $(HT_PARDF + parfolder)
//		wave /z freq, EinstA, EngGS
//		
//		
//		setdatafolder cdf
//		wavestats /Q freq
//		ValDisplay valdisp_numLinespar win = $PanelName, Value = _NUM:(V_npnts)
//		ValDisplay valdisp_parlowf win = $PanelName, Value = _NUM:(V_min)
//		ValDisplay valdisp_parhighf win = $PanelName, Value = _NUM:(V_max)
//	
//
//		make /free /b /n = (numpnts(freq)) logical
//		logical = (EinstA == 0)
//		variable missingEA = sum(logical)
//		ValDisplay valdisp_numNoApar win = $PanelName, Value = _NUM:(missingEA)
//
//		logical = (EngGS == -1)
//		variable missingEngGS = sum(logical)
//		ValDisplay valdisp_numNoGSpar win = $PanelName, Value = _NUM:(missingEngGS)
//
//		NVAR lowf = root:Packages:HITRAN:recentfreqlow
//		NVAR highf = root:Packages:HITRAN:recentfreqhigh
//		SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
//		lowf = V_min
//		highf = V_max
////		SetVariable setvar_lowf win = $PanelName, Value = _NUM:(V_min)
////		SetVariable setvar_highf win = $PanelName, Value = _NUM:(V_max)
//		CA_PanelUpdate(PanelName=PanelName)
//		S_recentparfolder = parfolder
	endif

	return 0
End

static Function CA_SelParSet(parfolder, [PanelName])
	String parfolder
	String PanelName
	if (paramisdefault(PanelName))
		PanelName = "PanelAdvisor"
	endif
	
	DFREF cdf = GetDataFolderDFR()
	setdatafolder $(HT_PARDF + parfolder)
	wave /z freq, EinstA, EngGS
	setdatafolder cdf
	wavestats /Q freq
	ValDisplay valdisp_numLinespar win = $PanelName, Value = _NUM:(V_npnts)
	ValDisplay valdisp_parlowf win = $PanelName, Value = _NUM:(V_min)
	ValDisplay valdisp_parhighf win = $PanelName, Value = _NUM:(V_max)
	

	make /free /b /n = (numpnts(freq)) logical
	logical = (EinstA == 0)
	variable missingEA = sum(logical)
	ValDisplay valdisp_numNoApar win = $PanelName, Value = _NUM:(missingEA)

	logical = (EngGS == -1)
	variable missingEngGS = sum(logical)
	ValDisplay valdisp_numNoGSpar win = $PanelName, Value = _NUM:(missingEngGS)

	NVAR lowf = root:Packages:HITRAN:recentfreqlow
	NVAR highf = root:Packages:HITRAN:recentfreqhigh
	SVAR S_recentparfolder = root:Packages:HITRAN:S_recentparfolder
	lowf = V_min
	highf = V_max
	CA_PanelUpdate(PanelName=PanelName)
	S_recentparfolder = parfolder
End

static Function CA_ButtonProc_SetAtmopshere(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	if (ba.eventCode == 2)			// mouse up
		HITRAN#SA_MainPanel()
	endif
	return 0
End

// Static Function HT_CalcAdvisorPanelUpdate([PanelName])
static Function CA_PanelUpdate([PanelName])
	string PanelName
	if (paramisdefault(PanelName))
		PanelName = "PanelAdvisor"
	endif
	ControlInfo /W=$PanelName popup_parset
	string parfolder = S_Value
	ControlInfo /W=$PanelName setvar_pressure
	variable pressure = V_Value
	ControlInfo /W=$PanelName setvar_temp
	variable temp = V_Value
	ControlInfo /W=$PanelName setvar_lowf
	variable freqlow = V_Value
	ControlInfo /W=$PanelName setvar_highf
	variable freqhigh = V_Value
	
	if (HT_CalcVarLineData(parfolder, pressure, temp, 3, 3) == 2)			
		Button button_SetAtmosphere win = $PanelName, fColor=(65280,0,0)		// 0 intensity calculated, update atmosphere
	else
		Button button_SetAtmosphere win = $PanelName, fColor=(65535,65535,65535)
	endif
	
	
	DFREF cdf = GetDataFolderDFR()
	setdatafolder $(HT_PARDF + parfolder)
	wave /z freq, EinstA, EngGS, GaussWidth, LorWidth
	setdatafolder cdf

	variable lowidx = BinarySearch(freq,freqlow)
	variable highidx = BinarySearch(freq,freqhigh)
	highidx = (highidx == -2) ? numpnts(freq) - 1 : highidx		// outofrange high with last point
	variable numlines
	if ((lowidx == -2) || (highidx == -1) || (lowidx == -3))
		numlines = 0
	else
		numlines = highidx - lowidx
	endif
	
	WaveStats  /Q /R = [lowidx, highidx] LorWidth
	ValDisplay valdisp_minLwidth  win =  $PanelName, value = _NUM:(V_Min)
	WaveStats  /Q /R = [lowidx, highidx] GaussWidth
	ValDisplay valdisp_minDwidth  win =  $PanelName, value = _NUM:(V_Min)
	
	
	variable dwidth = CalcAverageLineWidth(parfolder, 0, freqlow, freqhigh)
	variable lwidth = CalcAverageLineWidth(parfolder, 1, freqlow, freqhigh)
	variable recpointspace = sqrt(dwidth^2 + lwidth^2) / 2
	variable recwavesize = ceil((freqhigh - freqlow) / recpointspace) + 1
	make /free /b /n = (numpnts(freq)) logical
	logical = (EinstA == 0)
	variable missingEA_range = sum(logical,lowidx, highidx)
	logical = (EngGS == -1)
	variable missingEngGS_range = sum(logical,lowidx,highidx)

	// update the ranged information
	ValDisplay valdisp_numLinesrange win =  $PanelName, value = _NUM:(numlines)
	ValDisplay valdisp_numNoGSrange  win =  $PanelName, value = _NUM:(missingEngGS_range)
	ValDisplay valdisp_numNoArange  win =  $PanelName, value = _NUM:(missingEA_range)

	ValDisplay valdisp_avgLwidth  win =  $PanelName, value = _NUM:(lwidth)
	ValDisplay valdisp_avgDwidth  win =  $PanelName, value = _NUM:(dwidth)

	ValDisplay valdisp_RecWnSpace  win =  $PanelName, value = _NUM:(recpointspace)
	NVAR recentwnspacing = root:Packages:HITRAN:recentwnspacing
	recentwnspacing = recpointspace
//	ValDisplay valdisp_outputsize  win =  $PanelName, value = _NUM:(recwavesize*8 / 1e6)
	ValDisplay valdisp_outputsize  win =  $PanelName, value = _NUM:(recwavesize)
	ValDisplay valdisp_numcalcsaccurate win =  $PanelName, value = _NUM:(recwavesize*numlines)	// not quite right, need total lines in set, not in range
End



// calculates intensity weighted average linewidth over a range of base frequencies
// Assumes pressure and temperature effects have already been applied 
// (ModifyLorCoefs or ModifyVoightCoefs)

Static Function CalcAverageLineWidth(parfolder, whichwidth, freqlow, freqhigh)
	string parfolder
	variable whichwidth
	variable freqlow, freqhigh
//	parfolder = ParFolderNameParser(parfolder)
	string cdf = GetDataFolder(1)
	setdatafolder $(HT_PARDF + parfolder)
	wave /z VarFreq, VarInten
	if (whichwidth)				// Which width = 1 for lorentzian(pressure), 0 for gaussian(doppler)
		wave /z Width = LorWidth
	else
		wave /z Width = GaussWidth
	endif
	setdatafolder cdf
	if (!waveexists(Width) || !waveexists(VarFreq) || !waveexists(VarInten))
		return NaN
	endif
//	duplicate /FREE Var_Freq WeightedWidth
	matrixop /FREE RangedInten = VarInten * (greater(VarFreq,freqlow) && greater(freqhigh,VarFreq))
	matrixop /FREE MeanWeigthedWidth = RangedInten . Width / sum(RangedInten)
	return MeanWeigthedWidth[0]
End


Static Function PlotLinesMenu()
	string parfolder = ""
	variable PlotOn = 1
	variable AutoLegend = 1
	prompt parfolder, "Select Parameters Folder", popup, ParFolderList()
//	string Ydata = "VarInten"
//	prompt Ydata, "Y data", popup, "VarInten;VarEmit"
	prompt PlotOn, "Plot Lines On ", popup, "New Graph;Top Graph (left axis);Top Graph (right axis)"
	prompt AutoLegend "Molecule Color Legend", popup, "Yes; No"
	DoPrompt "Plot Line Data", parfolder, PlotOn, AutoLegend
	if (V_flag)
		return 0
	endif
	HT_PlotLines(parfolder, PlotOn, AutoLegend)
	return 0
End

Static Function HT_PlotLines(parfolder, PlotOn, AutoLegend)
	string parfolder
	variable PlotOn, AutoLegend
	string cdf = getdatafolder(1)
	setdatafolder $(HT_PARDF + parfolder)
	wave /z VarInten
	if (!waveexists(VarInten))			// Create Auxiliary Waves if not already created
//		ModifyLorCoefs(parfolder,STANDARDPRESS,STANDARDTEMP,3)
		wave Intensity, Freq
		duplicate Intensity VarInten
		duplicate /o Freq, VarFreq
	endif
	if (PlotOn == 1)
		Display
	Endif
	if (PlotOn == 3)
		AppendToGraph /R VarInten vs VarFreq
	else
		AppendToGraph VarInten vs VarFreq	
	endif
	wave molnum
//	display VarInten vs VarFreq
//	ModifyGraph zColor(VarInten)={molnum,1,NUMMOLECULES,$CT_FOR_MOLECULES,0}
	ModifyGraph zColor(VarInten)={molnum,*,*,cindexRGB,0,root:Packages:HITRAN:HT_ctabmol}		// this should work.  Igor bug? 
	ModifyGraph mode(VarInten)=1
	setdatafolder cdf
	if (AutoLegend == 1)
		LinePlotLegendGen(parfolder,0)
	endif
	setdatafolder cdf
End

// For use with a sticks to zero line data plot.  Calculates a bandsum (based on full range) at last run
// conditions (from VarInten vs. VarFreq) and generates a Legend sorted by most to least prevalent 
// molecules.
// howmany controls how many species to include, and if set to less than one uses an plots only those 
// species with line strengths within 1e6 of the strongest. 
 
Static Function LinePlotLegendGen(parfolder,howmany)
	string parfolder
	variable howmany
	wave /t moleculenames = root:Packages:HITRAN:molparamdata:molnum_code
	wave HT_ctabmol = root:Packages:HITRAN:HT_ctabmol
//	parfolder = ParFOlderNameParser(parfolder)
	DFREF cdf = getdatafolderDFR()
	setdatafolder $(HT_PARDF + parfolder)
	duplicate /free moleculenames molecules
	make /free /d /n = (numpnts(moleculenames)) inten_by_mol
	make /free /b /n = (numpnts(moleculenames)) molnums_sorted
	molnums_sorted = p
	wave molnum, VarInten
	variable cnt
	for (cnt = 0; cnt < numpnts(moleculenames) ; cnt += 1)
		matrixop /free tempsum = sumCols(VarInten * equal(cnt, molnum))
		inten_by_mol[cnt] = tempsum[0]
	endfor
	sort /r inten_by_mol, inten_by_mol, molecules, molnums_sorted
	wave /t molnames = molecules		// duplicate doesn't keep track of text status.
	string legendtext = ""
	FindValue /I=(molnums_sorted[0]) molnum 
	legendtext += "\\s(VarInten[" + num2str(V_value) + "]) " + molnames[0]
	variable maxlines = (howmany < 1) ? numpnts(molnames) : howmany
	for (cnt = 1; cnt < maxlines; cnt +=1 )
		if ((inten_by_mol[cnt] == 0 ) || ((howmany < 1) && (inten_by_mol[cnt] < (inten_by_mol[0] / 1e6))))
			break
		endif
		FindValue /I=(molnums_sorted[cnt]) molnum 
		legendtext += "\r\\s(VarInten[" + num2str(V_value) + "]) " + molnames[cnt]
	endfor
	Legend /C /N=text_stickslegend /J legendtext
	setdatafolder cdf
	return 1
End

Static Function LinePlotLegendWrapper()
	variable howmany
	prompt howmany, "How Many Species to Include? (0 for auto)"
	DoPrompt "Line Plot Legend", howmany
	wave /z VarInten = TraceNameToWaveRef("","VarInten")
//	string S_info = TraceInfo("","VarInten",0)
	if (!WaveExists(VarInten))
		print "Cannot find VarInten trace in the top graph"
		return 0
	endif
	string parfolder = GetWavesDataFolder(VarInten,0)
	 LinePlotLegendGen(parfolder,howmany)
	 return 1
End

// Adds the intensity strengths of lines without pre-broadening them but with population effects
//.  Same unit as a HITRAN line intensy:
//  cm2 / molecule * cm-1
Function HT_BandSum(parfolder, LowWN, HighWN)
	string parfolder
	variable LowWN, HighWN
	if (!strlen(parfolder))
		prompt parfolder, "Select Parameters Folder", popup, ParFolderList()
		DoPrompt "HT_BandSum", parfolder
		if (V_flag)
			return 0
		endif
	endif
//	parfolder = ParFOlderNameParser(parfolder)
//	wave VarInten = $(parfolder):VarInten
	wave VarInten = $(HT_PARDF+parfolder + ":VarInten")
	wave Freq = $(HT_PARDF+parfolder + ":Freq")
	variable LowIdx = BinarySearch(Freq, LowWN) + 1
	variable HighIdx = BinarySearch(Freq, HighWN)
	HighIdx = (HighIdx == -2) ? numpnts(Freq) : Highidx
	return Sum(VarInten,LowIdx,HighIdx)
End


